
import "Turbine.Gameplay"
import "Turbine.UI"
import "Turbine.UI.Lotro"

import "Souru.Gibberish.Utils"
import "Souru.Gibberish.UI"

---------------------------------------------------------------------
--== reloader ===--
---------------------------------------------------------------------

local status  = 0
control_raideffects = Turbine.UI.Control()
 
function control_raideffects:Update()
	if status == 1 then
        Turbine.PluginManager.UnloadScriptState( "GibberishReloader" )
    elseif status > 1 then
        self:SetWantsUpdates( false )
        
        control_raideffects = nil
	end

    status = status + 1
end

control_raideffects:SetWantsUpdates( true )


---------------------------------------------------------------------
--== global ===--
---------------------------------------------------------------------

EFFECTS_SELF = 1
EFFECTS_GROUP = 2
EFFECTS_TARGET = 3
CHAT = 4
SKILL = 5
TRIGGER_END = 6
TRIGGER_START = 8
TRIGGER_THRESHOLD = 7


lp = Turbine.Gameplay.LocalPlayer:GetInstance()
group = nil


moving = false
collect_effects = false
collect_target_effects = false
effects_target_only_buffs = true
collect_chat = false
effects_only_debuffs = true
chat_only_say = true
show_groups = true

windowdata = {}

windows = {}
groups = {}
logo = nil
settings_window = nil
collectors_window = nil
move_window = nil
move_selection = nil

library = {}
library["effects"] = {}
library["chat"] = {}

collection = {}
collection["effects"] = {}
collection["chat"] = {}

LIST_NAMES = {}
LIST_NAMES[1] = "list_effects_self"
LIST_NAMES[2] = "list_effects_group"
LIST_NAMES[3] = "list_effects_target"
LIST_NAMES[4] = "list_chat"
LIST_NAMES[5] = "list_skill"
LIST_NAMES[6] = "list_triggers_end"
LIST_NAMES[7] = "list_triggers_threshold"
LIST_NAMES[8] = "list_triggers_start"
--LIST_NAMES[9] = "all_self_debuffs_blacklist"

---------------------------------------------------------------------
--== saveFile localization ===--
-- by Ezuri Broda (Gwaihir) --
---------------------------------------------------------------------

locale = "en";
fileSuffix = "";
if Turbine.Shell.IsCommand("hilfe") then
	locale = "de";
	fileSuffix = "__de";
elseif Turbine.Shell.IsCommand("aide") then
	locale = "fr";
	fileSuffix = "__fr";
end

function convertToEuro(dataRaw)
	local newData = {};
	for i, myData in pairs(dataRaw) do
		local tempIndex = nil;
		local tempData = nil;
		if (type(i) == "number") then
			tempIndex = tostring(i);
		else
			tempIndex = i;
		end
		if (type(myData) == "table") then
			tempData = convertToEuro(myData);
		elseif (type(myData) == "number") then
			tempData = tostring(myData);
		else
			tempData = myData;
		end
		newData[tempIndex] = tempData;
	end
	return newData;
end

function convertFromEuro(dataRaw)
	local newData = {}
	for i, myData in pairs(dataRaw) do
		local tempIndex = tonumber(i);
		if (tempIndex == nil) then
			tempIndex = i;
		end
		local tempData = nil;

		if (type(myData) == "table") then
			tempData = convertFromEuro(myData);
		else
			tempData = tonumber(myData);
			if (tempData == nil) then
				tempData = myData;
			end
		end
		newData[tempIndex] = tempData;
	end
	return newData;
	
end


---------------------------------------------------------------------
--== saving ===--
---------------------------------------------------------------------

function SaveSettings()
	if locale == "en" then 
		Turbine.PluginData.Save(Turbine.DataScope.Character, "gibberish_char_file", windowdata)
		Turbine.PluginData.Save(Turbine.DataScope.Account, "gibberish_account_file", windowdata)
	else
		local NewSaveFile1 = convertToEuro(windowdata);
		local NewSaveFile2 = convertToEuro(windowdata);
		Turbine.PluginData.Save(Turbine.DataScope.Character, "gibberish_char_file" .. fileSuffix, NewSaveFile1)
		Turbine.PluginData.Save(Turbine.DataScope.Account, "gibberish_account_file" .. fileSuffix, NewSaveFile2)	
	end

end

function SaveLibrary()

	if locale == "en" then 
		Turbine.PluginData.Save(Turbine.DataScope.Account, "gibberish_library", library)
	else
		local NewSaveFile1 = convertToEuro(library);
		Turbine.PluginData.Save(Turbine.DataScope.Account, "gibberish_library" .. fileSuffix, NewSaveFile1)	
	end

end

Turbine.Plugin.Unload = function()

	UnloadSave("Gibberish unload.")

end

---------------------------------------------------------------------
--== misc ===--
---------------------------------------------------------------------

function UnloadSave(message)
	local list = {}

	for i,windata in ipairs(windowdata) do 

		if (windata["load"]) then
			if windows[i]~= nil then
				list[i] =  windows[i]:GetRunningTriggerList()
			end
		end

	end

	if locale == "en" then
		Turbine.PluginData.Save(Turbine.DataScope.Character, "gibberish_running_triggerlist", list)
	else 
		local NewSavelist = convertToEuro(list)
		Turbine.PluginData.Save(Turbine.DataScope.Character, "gibberish_running_triggerlist" .. fileSuffix, NewSavelist)
	end


	OptionsReload()

	--settings_Close()
	-- if moving then
	-- 	window_MovingChanged()
	-- end

	if message ~= nil then
		Turbine.Shell.WriteLine(message)
	end
	
	SortWindows()
end

function ReloadPlugin(message)


        Turbine.PluginManager.LoadPlugin( "GibberishReloader" )

end

function OptionsReload()

	local temp = {}
	if settings_window == nil then
		temp["options_open"] = false
	else
		temp["options_open"] = true

		if settings_window.selected_window_index ~= nil then
			temp["options_windowselection"] = settings_window.selected_window_index
		else
			temp["options_windowselection"] = false
		end

		if settings_window.trigger_window ~= nil then
			local temp_data = settings_window.trigger_window.data

		else
			temp["options_trigger"] = false
		end
	end

	
	temp["move_windows"] = moving
	temp["show_groups"] = show_groups

	if collectors_window == nil then
		temp["collection_open"] = false
	else
		temp["collection_open"] = true
	end


	if locale == "en" then
		Turbine.PluginData.Save(Turbine.DataScope.Character, "gibberish_optionwindow_state", temp)
	else 
		local NewSavelist = convertToEuro(temp)
		Turbine.PluginData.Save(Turbine.DataScope.Character, "gibberish_optionwindow_state" .. fileSuffix, NewSavelist)
	end

end

function SettingsOverwrite(data, parent, key, start_time, duration, icon, text, running)

    if(data["timer"] > 0) then
        duration = data["timer"]
    end

    if(data["text"] ~= nil and data["text"] ~= "%d") then
        text = data["text"]
    end

    if(data["icon"] ~= nil)  then
        icon = data["icon"]
    end

    return parent, key, start_time, duration, icon, text, data, running

end

function ColorToString(color)
	if color ~= nil then
    	return (math.round(color["R"]*255))..", "..(math.round(color["G"]*255))..", "..(math.round(color["B"]*255))
	end
end

function StringToColor(text)

    text = string.gsub(text, "\n", "")
    text = string.gsub(text, " ", "")
    local color_list = split(text, ",")

    if table.getn(color_list) == 3 then
        return color_list[1]/255, color_list[2]/255, color_list[3]/255
    else
        return nil
    end

end

function SecondsToClock(seconds)

    local seconds = tonumber(seconds)

    if seconds <= 0 then
        return "00:00";
    else
    --  hours = string.format("%02.f", math.floor(seconds/3600));
        mins = string.format("%02.f", math.floor(seconds/60));
        secs = string.format("%02.f", math.floor(seconds - mins *60));
        return mins..":"..secs
    end
end

function split(s, delimiter)
    result = {};
    for match in (s..delimiter):gmatch("(.-)"..delimiter) do
        table.insert(result, match);
    end
    return result;
end

findfont = function(number)
    if number == 1 then
        return Turbine.UI.Lotro.Font.Arial12;
    elseif number == 2 then
        return Turbine.UI.Lotro.Font.TrajanPro13;
    elseif number == 3 then
        return Turbine.UI.Lotro.Font.TrajanPro14;
    elseif number == 4 then
        return Turbine.UI.Lotro.Font.TrajanPro15;
    elseif number == 5 then
        return Turbine.UI.Lotro.Font.TrajanPro16;
    elseif number == 6 then
        return Turbine.UI.Lotro.Font.TrajanPro18;
    elseif number == 7 then
        return Turbine.UI.Lotro.Font.TrajanPro19;
    elseif number == 8 then
        return Turbine.UI.Lotro.Font.TrajanPro20;
    elseif number == 9 then
        return Turbine.UI.Lotro.Font.TrajanPro21;
    elseif number == 10 then
        return Turbine.UI.Lotro.Font.TrajanPro23;
    elseif number == 11 then
        return Turbine.UI.Lotro.Font.TrajanPro24;
    elseif number == 12 then
        return Turbine.UI.Lotro.Font.TrajanPro25;
    elseif number == 13 then
        return Turbine.UI.Lotro.Font.TrajanPro26;
    elseif number == 14 then
        return Turbine.UI.Lotro.Font.TrajanPro28;
    elseif number == 15 then
        return Turbine.UI.Lotro.Font.TrajanProBold16;
    elseif number == 16 then
        return Turbine.UI.Lotro.Font.TrajanProBold22;
    elseif number == 17 then
        return Turbine.UI.Lotro.Font.TrajanProBold24;
    elseif number == 18 then
        return Turbine.UI.Lotro.Font.TrajanProBold25;
    elseif number == 19 then
        return Turbine.UI.Lotro.Font.TrajanProBold30;
    elseif number == 20 then
        return Turbine.UI.Lotro.Font.TrajanProBold36;
    elseif number == 21 then
        return Turbine.UI.Lotro.Font.Verdana10;
    elseif number == 22 then
        return Turbine.UI.Lotro.Font.Verdana12;
    elseif number == 23 then
        return Turbine.UI.Lotro.Font.Verdana14;
    elseif number == 24 then
        return Turbine.UI.Lotro.Font.Verdana16;
    elseif number == 25 then
        return Turbine.UI.Lotro.Font.Verdana18;
    elseif number == 26 then
        return Turbine.UI.Lotro.Font.Verdana20;
    elseif number == 27 then
        return Turbine.UI.Lotro.Font.Verdana22;
    elseif number == 28 then
        return Turbine.UI.Lotro.Font.Verdana23;
    end
end

skill_tree_changed_control = Turbine.UI.Control()
skill_tree_changed_control.Update = function()
    if Turbine.Engine.GetGameTime() > skill_tree_changed_control.sleepEnd then
        skill_tree_changed_control:SetWantsUpdates(false)
        window_DataChanged()
		Trigger.checkAllSkills()
		Trigger.CheckAllActivEffects()
    end
end

function skill_tree_changed()
    
    skill_tree_changed_control.sleepEnd = Turbine.Engine.GetGameTime() + 3
    skill_tree_changed_control:SetWantsUpdates(true)

end


function TrimArticles(name)
	name = string.gsub(name, "^[Dd]ie ", "");
	name = string.gsub(name, "^[Dd]em ", "");
	name = string.gsub(name, "^[Dd]er ", "");
	name = string.gsub(name, "^[Dd]en ", "");
	return name;
end

function trim_articles(name)
	if (name == nil) then
		return nil;
	end

	-- Articles possibles: Mayara, LeMayara, Le Mayara, LaMayara, La Mayara, L' Mayara, L’Mayara, et peut être d'autres?
	return string.gsub(name, "^[Ll].-(%u)", "%1");
end

--CA parse-function

ParseCombatChat = {}

ParseCombatChat["en"] = function(line)

	-- 1) Damage line ---
	
	local initiatorName,avoidAndCrit,skillName,targetNameAmountAndType = string.match(line,"^(.*) scored a (.*)hit(.*) on (.*)%.$"); -- (updated in v4.1.0)
	
	if (initiatorName ~= nil) then
		
		initiatorName = string.gsub(initiatorName,"^[Tt]he ","");
		
		local avoidType =
			string.match(avoidAndCrit,"^partially blocked") and 8 or
			string.match(avoidAndCrit,"^partially parried") and 9 or
			string.match(avoidAndCrit,"^partially evaded") and 10 or 1;
		local critType =
			string.match(avoidAndCrit,"critical $") and 2 or
			string.match(avoidAndCrit,"devastating $") and 3 or 1;
		skillName = string.match(skillName,"^ with (.*)$") or L.DirectDamage; -- (as of v4.1.0)
		
		local targetName,amount,dmgType,moralePower = string.match(targetNameAmountAndType,"^(.*) for ([%d,]*) (.*)damage to (.*)$");
		-- damage was absorbed
		if targetName == nil then
			targetName = string.gsub(targetNameAmountAndType,"^[Tt]he ","");
			amount = 0;
			dmgType = 13;
			moralePower = 3;
		-- some damage was dealt
		else
			targetName = string.gsub(targetName,"^[Tt]he ","");
			amount = string.gsub(amount,",","")+0;
      
      dmgType = string.match(dmgType, "^%(.*%) (.*)$") or dmgType; -- 4.2.3 adjust for mounted combat
			-- note there may be no damage type
			dmgType = 
				dmgType == "Common " and 1 or
				dmgType == "Fire " and 2 or
				dmgType == "Lightning " and 3 or
				dmgType == "Frost " and 4 or
				dmgType == "Acid " and 5 or
				dmgType == "Shadow " and 6 or
				dmgType == "Light " and 7 or
				dmgType == "Beleriand " and 8 or
				dmgType == "Westernesse " and 9 or
				dmgType == "Ancient Dwarf-make " and 10 or 
        dmgType == "Orc-craft " and 11 or
        dmgType == "Fell-wrought " and 12 or 13;
			moralePower = (moralePower == "Morale" and 1 or moralePower == "Power" and 2 or 3);
		end
		
		-- Currently ignores damage to power
		if (moralePower == 2) then return nil end
		
		-- Update
		return 1,initiatorName,targetName,skillName,amount,avoidType,critType,dmgType;
	end
	
	-- 2) Heal line --
	--     (note the distinction with which self heals are now handled)
	--     (note we consider the case of heals of zero magnitude, even though they presumably never occur)
	local initiatorName,crit,skillNameTargetNameAmountAndType = string.match(line,"^(.*) applied a (.-)heal (.*)%.$");
	
	if (initiatorName ~= nil) then
		initiatorName = string.gsub(initiatorName,"^[Tt]he ","");
		local critType =
			crit == "critical " and 2 or
			crit == "devastating " and 3 or 1;
		
		local skillNameTargetNameAndAmount,ending = string.match(skillNameTargetNameAmountAndType,"^(.*)to (.*)$");
		local targetName,skillName,amount;
		moralePower = (ending == "Morale" and 1 or (ending == "Power" and 2 or 3));
		-- heal was absorbed (unfortunately it appears this actually shows as a "hit" instead, so we never get into the first conditional)
		if (moralePower == 3) then
			targetName = string.gsub(ending,"^[Tt]he ","");
			amount = 0;
			-- skill name will equal nil if this was a self heal
			skillName = string.match(skillNameTargetNameAndAmount,"^with (.*) $");
		-- heal applied
		else
			skillName,targetName,amount = string.match(skillNameTargetNameAndAmount,"^(.*)to (.*) restoring ([%d,]*) points? $");
			targetName = string.gsub(targetName,"^[Tt]he ","");
			amount = string.gsub(amount,",","")+0;
			-- skill name will equal nil if this was a self heal
			skillName = string.match(skillName,"^with (.*) $");
		end
		
		-- rearrange if this was a self heal
		if (skillName == nil) then
			skillName = initiatorName;
			initiatorName = targetName;
		end
		
		-- Update
		return (moralePower == 2 and 4 or 3),initiatorName,targetName,skillName,amount,critType;
	end
	
	-- 3) Buff line --
	
	local initiatorName,skillName,targetName = string.match(line,"^(.*) applied a benefit with (.*) on (.*)%.$");
	
	if (initiatorName ~= nil) then
		initiatorName = string.gsub(initiatorName,"^[Tt]he ","");
		targetName = string.gsub(targetName,"^[Tt]he ","");
		
		-- Update
		return 17,initiatorName,targetName,skillName;
	end
	
	-- 4) Avoid line --
	local initiatorNameMiss,skillName,targetNameAvoidType = string.match(line,"^(.*) to use (.*) on (.*)%.$");
	
	if (initiatorNameMiss ~= nil) then
		initiatorName = string.match(initiatorNameMiss,"^(.*) tried$");
		local targetName, avoidType;
		-- standard avoid
		if (initiatorName ~= nil) then
			initiatorName = string.gsub(initiatorName,"^[Tt]he ","");
			targetName,avoidType = string.match(targetNameAvoidType,"^(.*) but (.*) the attempt$");
			targetName = string.gsub(targetName,"^[Tt]he ","");
			avoidType = 
				string.match(avoidType," blocked$") and 5 or
				string.match(avoidType," parried$") and 6 or
				string.match(avoidType," evaded$") and 7 or
				string.match(avoidType," resisted$") and 4 or
				string.match(avoidType," was immune to$") and 3 or 1;
				
		-- miss or deflect (deflect added in v4.2.2)
		else
			initiatorName = string.match(initiatorNameMiss,"^(.*) missed trying$");
      if (initiatorName == nil) then
        initiatorName = string.match(initiatorNameMiss,"^(.*) was deflected trying$");
        avoidType = 11;
      else
        avoidType = 2;
      end
      
			initiatorName = string.gsub(initiatorName,"^[Tt]he ","");
			targetName = string.gsub(targetNameAvoidType,"^[Tt]he ","");
		end
		
		-- Sanity check: must have avoided in some manner
		if (avoidType == 1) then return nil end
		
		-- Update
		return 1,initiatorName,targetName,skillName,0,avoidType,1,13;
	end
	
	-- 5) Reflect line --
	
	local initiatorName,amount,dmgType,targetName = string.match(line,"^(.*) reflected ([%d,]*) (.*) to the Morale of (.*)%.$");
	
	if (initiatorName ~= nil) then
		local skillName = "Reflect";
		initiatorName = string.gsub(initiatorName,"^[Tt]he ","");
		targetName = string.gsub(targetName,"^[Tt]he ","");
		amount = string.gsub(amount,",","")+0;
		
		local dmgType = string.match(dmgType,"^(.*)damage$");
		-- a damage reflect
		if (dmgType ~= nil) then
			dmgType = 
				dmgType == "Common " and 1 or
				dmgType == "Fire " and 2 or
				dmgType == "Lightning " and 3 or
				dmgType == "Frost " and 4 or
				dmgType == "Acid " and 5 or
				dmgType == "Shadow " and 6 or
				dmgType == "Light " and 7 or
				dmgType == "Beleriand " and 8 or
				dmgType == "Westernesse " and 9 or
				dmgType == "Ancient Dwarf-make " and 10 or
				dmgType == "Orc-craft " and 11 or
				dmgType == "Fell-wrought " and 12 or 13;
						
			-- Update
			return 1,initiatorName,targetName,skillName,amount,1,1,dmgType;
		-- a heal reflect
		else
			-- Update
			return 3,initiatorName,targetName,skillName,amount,1;
		end
	end
	
	-- 6) Temporary Morale bubble line (as of 4.1.0)
  local amount = string.match(line,"^You have lost ([%d,]*) points of temporary Morale!$");
	if (amount ~= nil) then
		amount = string.gsub(amount,",","")+0;
		
		-- the only information we can extract directly is the target and amount
		return 14,nil,player.name,nil,amount;
	end
	
	-- 7) Combat State Break notice (as of 4.1.0)
	
	-- 7a) Root broken
	local targetName = string.match(line,"^.* ha[sv]e? released (.*) from being immobilized!$");
	if (targetName ~= nil) then
		targetName = string.gsub(targetName,"^[Tt]he ","");
		
		-- the only information we can extract directly is the target name
		return 16,nil,targetName,nil;
	end
	
	-- 7b) Daze broken
	local targetName = string.match(line,"^.* ha[sv]e? freed (.*) from a daze!$");
	if (targetName ~= nil) then
		targetName = string.gsub(targetName,"^[Tt]he ","");
		
		-- the only information we can extract directly is the target name
		return 16,nil,targetName,nil;
	end
	
	-- 7c) Fear broken (TODO: Check)
	local targetName = string.match(line,"^.* ha[sv]e? freed (.*) from a fear!$");
	if (targetName ~= nil) then
		targetName = string.gsub(targetName,"^[Tt]he ","");
		
		-- the only information we can extract directly is the target name
		return 16,nil,targetName,nil;
	end
	
	-- 8) Interrupt line --
	
	local targetName, initiatorName = string.match(line,"^(.*) was interrupted by (.*)!$");
	
	if (targetName ~= nil) then
		initiatorName = string.gsub(initiatorName,"^[Tt]he ","");
		targetName = string.gsub(targetName,"^[Tt]he ","");
		
		-- Update
		return 7,initiatorName,targetName;
	end
	
	-- 9) Dispell line (corruption removal) --
	
	local corruption, targetName = string.match(line,"You have dispelled (.*) from (.*)%.$");
	
	if (corruption ~= nil) then
		initiatorName = player.name;
		targetName = string.gsub(targetName,"^[Tt]he ","");
		
		-- NB: Currently ignore corruption name
		
		-- Update
		return 8,initiatorName,targetName;
	end
	
	-- 10) Defeat lines ---
	
	-- 10a) Defeat line 1 (mob or played was killed)
	local initiatorName = string.match(line,"^.* defeated (.*)%.$");
	
	if (initiatorName ~= nil) then
		initiatorName = string.gsub(initiatorName,"^[Tt]he ","");
		
		-- Update
		return 9,initiatorName;
	end
	
	-- 10b) Defeat line 2 (mob died)
	local initiatorName = string.match(line,"^(.*) died%.$");

	if (initiatorName ~= nil) then
		initiatorName = string.gsub(initiatorName,"^[Tt]he ","");
		
		-- Update
		return 9,initiatorName;
	end
	
	-- 10c) Defeat line 3 (a player was killed or died)
	local initiatorName = string.match(line,"^(.*) has been defeated%.$");

	if (initiatorName ~= nil) then
		initiatorName = string.gsub(initiatorName,"^[Tt]he ","");
		
		-- Update
		return 9,initiatorName;
	end
	
	-- 10d) Defeat line 4 (you were killed)
	local match = string.match(line,"^.* incapacitated you%.$");

	if (match ~= nil) then
		initiatorName = player.name;
		
		-- Update
		return 9,initiatorName;
	end
	
	-- 10e) Defeat line 5 (you died)
	local match = string.match(line,"^You have been incapacitated by misadventure%.$");

	if (match ~= nil) then
		initiatorName = player.name;
		
		-- Update
		return 9,initiatorName;
	end
	
	-- 10f) Defeat line 6 (you killed a mob)
	local initiatorName = string.match(line,"^Your mighty blow topples (.*)%.$");
	
	if (initiatorName ~= nil) then
		initiatorName = string.gsub(initiatorName,"^[Tt]he ","");
		
		-- Update
		return 9,initiatorName;
	end
	
	-- 11) Revive lines --
	
	-- 11a) Revive line 1 (player revived)
	local initiatorName = string.match(line,"^(.*) has been revived%.$");
	
	if (initiatorName ~= nil) then
	  initiatorName = string.gsub(initiatorName,"^[Tt]he ","");
	  
		-- Update
	  return 10,initiatorName;
	end
	
	-- 11b) Revive line 2 (player succumbed)
	local initiatorName = string.match(line,"^(.*) has succumbed to .* wounds%.$");
	
	if (initiatorName ~= nil) then
	  initiatorName = string.gsub(initiatorName,"^[Tt]he ","");
	  
		-- Update
	  return 10,initiatorName;
	end
	
	-- 11c) Revive line 3 (you were revived)
	local match = string.match(line,"^You have been revived%.$");
	
	if (match ~= nil) then
	  initiatorName = player.name;
	  
		-- Update
	  return 10,initiatorName;
	end
	
	-- 11d) Revive line 4 (you succumbed)
	local match = string.match(line,"^You succumb to your wounds%.$");
	
	if (match ~= nil) then
	  initiatorName = player.name;
	  
		-- Update
	  return 10,initiatorName;
	end
	
	-- if we reach here, we were unable to parse the line
	--  (note there is very little that isn't parsed)
end


ParseCombatChat["de"] = function(line)

	-- 1) Damage line ---
	
	local initiatorName,avoidAndCrit,skillName,targetNameAmountAndType = string.match(line,"^(.*) gelang ein (.*)Treffer mit \"(.*)\" gegen (.*)%.$");
	
	if (initiatorName ~= nil) then
		initiatorName = TrimArticles(initiatorName);
		
		local avoidType =
			string.match(avoidAndCrit,"^teilweise geblockter") and 8 or
			string.match(avoidAndCrit,"^teilweise parierter") and 9 or
			string.match(avoidAndCrit,"^teilweise ausgewichener") and 10 or 1;
		local critType =
			string.match(avoidAndCrit,"kritischer $") and 2 or
			string.match(avoidAndCrit,"zerst\195\182rerischer $") and 3 or 1;
		
		local targetName,amount,dmgType,moralePower = string.match(targetNameAmountAndType, "^(.*) f\195\188r ([%d,]*) Punkte Schaden des Typs \"(.*)\" auf (.*)$");
		-- damage was absorbed
		if targetName == nil then
			targetName = TrimArticles(targetNameAmountAndType);
			amount = 0;
			dmgType = 13;
			moralePower = 3;
		-- some damage was dealt
		else
			targetName = TrimArticles(targetName);
			amount = string.gsub(amount,",","")+0;
			-- note there may be no damage type
			dmgType = 
				dmgType == "Allgemein" and 1 or
				dmgType == "Feuer" and 2 or
				dmgType == "Blitz" and 3 or
				dmgType == "Frost" and 4 or
				dmgType == "S\195\164ure" and 5 or
				dmgType == "Schatten" and 6 or
				dmgType == "Licht" and 7 or
				dmgType == "Beleriand" and 8 or
				dmgType == "Westernis" and 9 or
				dmgType == "Uralte Zwergenart" and 10 or
				dmgType == "Ork-Waffe" and 11 or
				dmgType == "Hass" and 12 or 13;
			moralePower = (moralePower == "Moral" and 1 or moralePower == "Kraft" and 2 or 3);
		end
		
		-- Currently ignores damage to power
		if (moralePower == 2) then return nil end
		
		-- Update
		return 1,initiatorName,targetName,skillName,amount,avoidType,critType,dmgType;
	end
	
	-- 2) Heal line --
	
	--     (note the distinction with which self heals are now handled)
	--     (note we consider the case of heals of zero magnitude, even though they presumably never occur)
	local initiatorName,crit,skillNameTargetNameAmountAndType = string.match(line,"^(.*) wandte \"(.*)Heilung\" mit (.*)%.$");

	if (initiatorName ~= nil) then
		initiatorName = TrimArticles(initiatorName);
		local critType =
			crit == "kritische " and 2 or
			crit == "zerst\195\182rerische " and 3 or 1;
			
		local skillNameTargetNameAndAmount,ending = string.match(skillNameTargetNameAmountAndType,"^(.*) Punkte (.*) wiederherstellte$");
		moralePower = (ending == "Moral" and 1 or ending == "Kraft" and 2 or 3);
		
		skillName,targetName,amount = string.match(skillNameTargetNameAndAmount,"^\"(.*)\" auf (.*) an, was ([%d,]*)$");
		targetName = TrimArticles(targetName);
		amount = string.gsub(amount,",","")+0;
		
		-- Update
		return (moralePower == 2 and 4 or 3),initiatorName,targetName,skillName,amount,critType;
	end
	
	-- 2.2) Self Heal
	
	--		(note that the self heal line is totally differend in comparision to the normal heal line in the german client)
	
	local skillName, initiatorName, critType, amount, moralPower = string.match(line, "^(.*) verursacht bei (.*) \"(.*)Heilung\" und stellt ([%d,]*) Punkte (.*) wieder her%.");
	if(initiatorName ~= nil) then
		initiatorName = TrimArticles(initiatorName);
		amount = string.gsub(amount, ",", "")+0;
		moralPower = (moralPower == "Moral" and 1 or moralPower == "Kraft" and 2 or 3);
		critType = critType == "kritische " and 2 or
				   critType == "zerst\195\182rerische " and 3 or 1;
		
		return (moralPower == 2 and 4 or 3), initiatorName, initiatorName, skillName, amount, critType;
	end
	
	-- 3) Buff line --
	local initiatorName,skillName,targetName = string.match(line,"^(.*) wandte \"Vorteil\" mit \"(.*)\" auf (.*) an%.$");
	
	if (initiatorName ~= nil) then
		initiatorName = TrimArticles(initiatorName);
		targetName = TrimArticles(targetName);
		-- Update
		return 17,initiatorName,targetName,skillName;
	end
	
	-- 4) Avoid line --


	-- standard avoid
	local initiatorName,targetName,skillName,erSie,avoidType = string.match(line,"^(.*) wollte (.*) mit (.*) treffen\, aber(.*)konterte den Versuch mit (.*).");
	if (initiatorName ~= nil) then
		initiatorName = TrimArticles(initiatorName);
		targetName = TrimArticles(targetName);
		skillName = string.gsub(skillName, "\"", "");
		avoidType = string.gsub(avoidType, "\"", "");
		avoidType = 
				string.match(avoidType,"Blocken") and 5 or
				string.match(avoidType,"Parieren") and 6 or
				string.match(avoidType,"Ausweichen") and 7 or
				string.match(avoidType,"Widerstehen") and 4 or
				string.match(avoidType,"Immunit\195\164t") and 3 or 1;
		-- Sanity check: must have avoided in some manner
		if (avoidType == 1) then return nil end
		return 1,initiatorName,targetName,skillName,0,avoidType,1,10;
	end
		
	-- miss
	local initiatorName, targetName, skillName = string.match(line, "^(.*) verfehlte (.*) mit (.*)%.");
	
	if (initiatorName ~= nil) then
		initiatorName = TrimArticles(initiatorName);
		targetName = TrimArticles(targetName);
		skillName = string.gsub(skillName, "\"", "");
		local avoidType = 2;
		-- Update
		return 1,initiatorName,targetName,skillName,0,avoidType,1,10;
	end
		
	
	-- 5) Reflect line --
	
	local initiatorName, amount, reflectType, targetName = string.match(line, "^(.*) reflektierte ([%d,]*) Punkte (.*) von (.*).$");
	if(initiatorName ~= nil) then
		local skillName = "Reflektiert";
		initiatorName = TrimArticles(initiatorName);
		targetName = TrimArticles(targetName);
		
		amount = string.gsub(amount,",","")+0;
		local dmgType = string.match(reflectType, "^Schaden des Typs (.*) auf Moral$");
		
		if(dmgType ~= nil) then
			dmgType = 
				dmgType == "\"Allgemein\""  and 1 or
				dmgType == "\"Feuer\"" and 2 or
				dmgType == "\"Blitz\"" and 3 or
				dmgType == "\"Frost\"" and 4 or
				dmgType == "\"S\195\164ure\"" and 5 or
				dmgType == "\"Schatten\"" and 6 or
				dmgType == "\"Licht\"" and 7 or
				dmgType == "\"Beleriand\"" and 8 or
				dmgType == "\"Westernis\"" and 9 or
				dmgType == "\"Uralte Zwergenart\"" and 10 or
				dmgType == "\"Ork-Waffe\"" and 11 or
				dmgType == "\"Hass\"" and 12 or 13;
			-- a dmg reflect
			return 1,initiatorName,targetName,skillName,amount,1,1,dmgType;
		else
			-- a heal reflect
			return 3,initiatorName,targetName,skillName,amount,1;
		end
	end
	
	-- 6) Temporary Morale bubble line (as of 4.1.0)
	local amount = string.match(line,"^Ihr habt ([%d,]*) Punkte Moral (tempor\195\164r) verloren!$");
	if (amount ~= nil) then
		amount = string.gsub(amount,",","")+0;
		-- the only information we can extract directly is the target and amount
		return 14,nil,player.name,nil,amount;
	end
	
	-- 7) Combat State Break notice (as of 4.1.0)
	
	-- 7a) Root broken
	local targetName = string.match(line,"^.* hab?t die Bewegungslosigkeit von (.*) beendet!$");
	if (targetName ~= nil) then
		targetName = TrimArticles(targetName);
		-- the only information we can extract directly is the target name
		return 16,nil,targetName,nil;
	end
	
	-- 7b) Daze broken
	local targetName = string.match(line,"^.* hab?t die Benommenheit von (.*) beendet!$");
	if (targetName ~= nil) then
		targetName = TrimArticles(targetName);
		-- the only information we can extract directly is the target name
		return 16,nil,targetName,nil;
	end
	
	-- 7c) Fear broken
	local targetName = string.match(line,"^.* hab?t mit einem Angriff die Furcht von (.*) beseitigt!$");
	if(targetName ~= nil) then
		targetName = TrimArticles(targetName);
		-- the only information we can extract directly is the target name
		return 16,nil,targetName,nil;
	end
		
	
	-- 8) Interrupt line --
	local targetName,initiatorName = string.match(line,"^(.*) wurde durch (.*) unterbochen!");
	if (targetName ~= nil) then
		initiatorName = TrimArticles(initiatorName);
		targetName = TrimArticles(targetName);
		
		-- Update
		return 7,initiatorName,targetName;
	end
	
	-- 9) Dispell line (corruption removal) --
	local corruption, targetName = string.match(line,"Von (.*) geheilt: (.*)");
	
	if (corruption ~= nil) then
		-- NB: Currently ignore corruption name
		
		-- Update
		return 8,player.name,targetName;
	end
	
	-- 10) Defeat lines ---
	
	-- 10a) Defeat line 1 (mob or played was killed)
	local initiatorName = string.match(line,"^Durch .* besiegt: (.*)%.$");
	
	if (initiatorName ~= nil) then
		initiatorName = TrimArticles(initiatorName);
		-- Update
		return 9,initiatorName;
	end
	
	-- 10b) Defeat line 2 (mob died)
	local initiatorName = string.match(line,"^(.*) ist gestorben%.$");
	
	if (initiatorName ~= nil) then
		initiatorName = TrimArticles(initiatorName);
		-- Update
		return 9,initiatorName;
	end
	
	-- 10c) Defeat line 3 (a player was killed or died)
	local initiatorName = string.match(line,"^(.*) wurde besiegt%.$");

	if (initiatorName ~= nil) then
		initiatorName = TrimArticles(initiatorName);
		-- Update
		return 9,initiatorName;
	end
	
	-- 10d) Defeat line 4 (you were killed)
	local match = string.match(line,"^.* hat Euch au\195\159er Gefecht gesetzt%.$");

	if (match ~= nil) then
		-- Update
		return 9,player.name;
	end
	
	-- 10e) Defeat line 5 (you died)
	local match = string.match(line,"^Ihr wurdet durch ein Missgeschick au\195\159er Gefecht gesetzt%.$");

	if (match ~= nil) then
		-- Update
		return 9,player.name;
	end
	
	-- 11) Revive lines --
	
	-- 11a) Revive line 1 (player revived)
	local initiatorName = string.match(line,"^(.*) wurde wiederbelebt%.$");
	
	if (initiatorName ~= nil) then
		initiatorName = TrimArticles(initiatorName);
		-- Update
	  return 10,initiatorName;
	end
	
	-- 11b) Revive line 2 (player succumbed)
	local initiatorName = string.match(line,"^(.*) ist .* Wunden erlegen%.$");
	
	if (initiatorName ~= nil) then
	  initiatorName = TrimArticles(initiatorName);
		-- Update
	  return 10,initiatorName;
	end
	
	-- 11c) Revive line 3 (you were revived)
	local match = string.match(line,"^Ihr wurdet wiederbelebt%.$");
	
	if (match ~= nil) then
		-- Update
	  return 10,player.name;
	end
	
	-- 11d) Revive line 4 (you succumbed)
	local match = string.match(line,"^Ihr erliegt Euren Verletzungen%.$");
	
	if (match ~= nil) then
		-- Update
	  return 10,player.name;
	end
	
	-- if we reach here, we were unable to parse the line
	--  (note there is very little that isn't parsed)
end

ParseCombatChat["fr"] = function(line)

	-- 1) Damage line ---
	
	local initiatorName,avoidAndCrit,skillName,targetName,amount,dmgType,moralePower = string.match(line,"^(.*) a infligé un coup (.*)avec (.*) sur (.*) pour ([%d,]*) points de type (.*) à l'entité ?(.*)%.$"); -- (updated in v4.1.0)
	
	if (initiatorName ~= nil) then
		
		local avoidType =
			string.match(avoidAndCrit,"partiellement bloqué") and 8 or
			string.match(avoidAndCrit,"partiellement paré") and 9 or
			string.match(avoidAndCrit,"partiellement esquivé ") and 10 or 
			1;
		local critType =
			string.match(avoidAndCrit,"critique") and 2 or
			string.match(avoidAndCrit,"dévastateur") and 3 or 1;
			
		-- skillName = string.match(skillName,"^ avec (.*)$") or L.DirectDamage; -- (as of v4.1.0)

    if (printDebug) then
       Turbine.Shell.WriteLine( "damage by "..initiatorName.." skill "..skillName );
    end		
		-- damage was absorbed
		if targetName == nil then
			targetName = string.gsub(targetNameAmountAndType,"^[Tt]he ","");
			amount = 0;
			dmgType = 12;
			moralePower = 3;
		-- some damage was dealt
		else
			amount = string.gsub(amount,",","")+0;
      
---			dmgType = string.match(dmgType, "^%(.*%) (.*)$") or dmgType; -- 4.2.3 adjust for mounted combat
			-- note there may be no damage type
			dmgType = 
				dmgType == "Commun" and 1 or
				dmgType == "Feu" and 2 or
				dmgType == "Foudre" and 3 or
				dmgType == "Froid" and 4 or
				dmgType == "Acide" and 5 or
				dmgType == "Ombre" and 6 or
				dmgType == "Lumière" and 7 or
				dmgType == "Beleriand" and 8 or
				dmgType == "Ouistrenesse" and 9 or
				dmgType == "de nain d'antan" and 10 or 
				dmgType == "Orc" and 11 or
				dmgType == "Fell-wrought" and 12 or 13;
			moralePower = (moralePower == "Morale" and 1 or moralePower == "Puissance" and 2 or 3);
		end
		
		-- Currently ignores damage to power
		if (moralePower == 2) then return nil end
		
		-- Update
		return 1,trim_articles(initiatorName),trim_articles(targetName),skillName,amount,avoidType,critType,dmgType;
	end
	
	-- 2) Heal line --
	--     (note the distinction with which self heals are now handled)
	--     (note we consider the case of heals of zero magnitude, even though they presumably never occur)
	
	--	[slfHeal] Arc du Juste a appliqué un soin critique à Ardichas, redonnant 52 points à l'entité Puissance.
	--  [slfHeal] Esprit de soliloque a appliqué un soin critique à Yogimen, redonnant 228 points à l'entité Moral.
	--  [Heal]    Eleria a appliqué un soin avec Paroles de guérison Ardicapde, redonnant 227 points à Moral.
	initiator_name, match = string.match(line, '^(.*) a appliqu\195\169 un soin (.*)%.$');

	if (initiator_name ~= nil) then
			crit_type =
				string.match(match, 'critique') and 2 or
				string.match(match, 'dévastateur') and 3 or
				1;
			match = string.gsub(match, '^critique ', '');
			match = string.gsub(match, '^dévastateur ', '');

		local self_heal = (string.match(match, '^\195\160 ') and true or false);

		-- Soins personnels (Self heal)
		if (self_heal) then
			skill_name = initiator_name;
			target_name, dmg_amount, morale_power = string.match(match, '^\195\160 (.*), redonnant ([%d,]*) points? \195\160 l\'entit\195\169 ?(.*)$');
			initiator_name = target_name;

		-- Soins sur une cible (Heal applied)
		else
			skill_name, target_name, dmg_amount, morale_power = string.match(match, '^avec (.*) ([^%s]+), redonnant ([%d,]*) points? \195\160 ?(.*)$');
		end

		morale_power = (morale_power == 'Moral' and 1 or (morale_power == 'Puissance' and 2 or 3));
		dmg_amount = (morale_power == 3 and 0 or string.gsub(dmg_amount, ',', '') + 0);

		return (morale_power == 2 and 4 or 3), trim_articles(initiator_name), trim_articles(target_name), skill_name, dmg_amount, crit_type;
	end
	
	-- 3) Buff line --
	-- MarieChantal a appliqué un bénéfice avec Paroles de guérison Eleria.

	local initiatorName,skillName,targetName = string.match(line,"^(.*) a appliqué un bénéfice avec (.*) (.*)%.$");
	
	if (initiatorName ~= nil) then
		
		-- Update
		return 17,trim_articles(initiatorName),trim_articles(targetName),skillName;
	end
	
	-- 4) Avoid line --
	-- L' Profanateur ghâshfra vigoureux a essayé d'utiliser une attaque de lancer faible sur MarieChantal mais a esquivé la tentative.
	-- L' Berserker hante-jours a essayé d'utiliser une double attaque au corps à corps sur Eleria mais elle a paré la tentative.
	-- L' Archer corsaire a essayé d'utiliser une attaque au corps à corps faible sur Cashel mais il a paré la tentative.
	-- Ardichas a essayé d'utiliser Tir pénétrant amélioré : Brûlure sur le Trompeur immonde vigoureux mais il a esquivé la tentative.

	-- 4a) Évitements complets (Full avoids)
	local initiator_name, skill_name, target_name, avoidType = string.match(line, "^(.*) a essayé d'utiliser (.*) sur (.*) mais (.*) la tentative%.$");

	if (initiator_name ~= nil) then
		avoid_Type =
			string.match(avoidType,"bloqué") and 5 or
			string.match(avoidType,"paré") and 6 or
			string.match(avoidType,"esquivé") and 7 or
			string.match(avoidType,"résisté") and 4 or
			string.match(avoidType,"immunisé contre") and 3 or 1;

		if (avoid_type == 1) then
			return nil;
		end
		return 1, trim_articles(initiator_name), trim_articles(target_name), skill_name, 0, avoid_Type, 1, 10;
	end
	
	-- 4b) miss or deflect (deflect added in v4.2.2)
	-- La Berserker hante-jours n'a pas réussi à utiliser une frappe de taille faible sur la Eleria.
	local initiator_name, skill_name, target_name = string.match(line, "^(.*) n'a pas réussi à utiliser (.*) sur (.*)%.$");

	if (initiator_name ~= nil) then
		avoid_type = 2;

		return 1, trim_articles(initiator_name), trim_articles(target_name), skill_name, 0, avoid_type, 1, 10;
	end
	
	-- 5) Reflect line --
	-- Norchador a renvoyé 210 Ombre de dégâts au Moral de la Eleria.
	-- Le Sangsue gardienne a renvoyé 339 points redonnés au Moral de Eleria.
	local initiatorName,amount,dmgType,targetName = string.match(line,"^(.*) a renvoyé ([%d,]*) (.*) au Moral de (.*)%.$");
	
	if (initiatorName ~= nil) then
		local skillName = "Reflect";
		amount = string.gsub(amount,",","")+0;
		
		local dmgType = string.match(dmgType,"^(.*)de dégâts$");
		-- a damage reflect
		if (dmgType ~= nil) then
			dmgType = 
				dmgType == "Commun" and 1 or
				dmgType == "Feu" and 2 or
				dmgType == "Foudre" and 3 or
				dmgType == "Froid" and 4 or
				dmgType == "Acide" and 5 or
				dmgType == "Ombre" and 6 or
				dmgType == "Lumière" and 7 or
				dmgType == "Beleriand" and 8 or
				dmgType == "Ouistrenesse" and 9 or
				dmgType == "Nain d'antan" and 10 or 11;
						
			-- Update
			return 1,trim_articles(initiatorName),trim_articles(targetName),skillName,amount,1,1,dmgType;
		-- a heal reflect
		else
			-- Update
			return 3,trim_articles(initiatorName),trim_articles(targetName),skillName,amount,1;
		end
	end
	
	-- 6) Temporary Morale bubble line (as of 4.1.0)
  local amount = string.match(line,"^Vous avez perdu ([%d,]*) points de Moral temporaire !$");
	if (amount ~= nil) then
		amount = string.gsub(amount,",","")+0;
		
		-- the only information we can extract directly is the target and amount
		return 14,nil,trim_articles(player.name),nil,amount;
	end
	
	-- 7) Combat State Break notice (as of 4.1.0)
	
	-- 7a) Root broken
	initiator_name, target_name = string.match(line, "^(.*) délivrée? ?(.*) de l'immobilisation\194\160!$");

	if (initiator_name ~= nil) then
		initiator_name =
			string.match(initiator_name, "^Vous avez") and player.name or
			string.match(initiator_name, " vous a$") and string.gsub(initiator_name, " vous a$", "") or
			string.gsub(initiator_name, " a$", "");
		target_name = (target_name == "" and player.name or target_name);

		if (printDebug) then
  		Turbine.Shell.WriteLine("root_broken", line, "ini_name: " .. initiator_name .. " tgt_name: " .. target_name);
		end

		return 16, nil, trim_articles(target_name), nil;
	end
	
	-- 7b) Daze broken
	initiator_name, target_name = string.match(line, "^(.*) délivrée? ?(.*) de l'hébétement\194\160!$");

	if (initiator_name ~= nil) then
		initiator_name =
			string.match(initiator_name, "^Vous avez") and player.name or
			string.match(initiator_name, " vous a$") and string.gsub(initiator_name, " vous a$", "") or
			string.gsub(initiator_name, " a$", "");
		target_name = (target_name == "" and player.name or target_name);

		if (printDebug) then
		  Turbine.Shell.WriteLine("daze_broken", line, "ini_name: " .. initiator_name .. " tgt_name: " .. target_name);
		end

		return 16, nil, target_name, nil;
	end
	
	-- 8) Interrupt line --
	
	local target_name, initiator_name = string.match(line, "^(.*) a été interrompu par (.*)!$");

	if (target_name ~= nil) then
		return 7, trim_articles(initiator_name), trim_articles(target_name);
	end
	
	-- 9) Dispell line (corruption removal) --
	
	local corruption, target_name = string.match(line, "Vous avez dissipé l'effet (.*) affectant (.*)%.$");

	if (corruption ~= nil) then
		initiator_name = player.name;
		-- NB: Currently ignore corruption name
		
		-- Update
		return 8, trim_articles(initiator_name), trim_articles(target_name);
	end
	
	-- 10) Defeat lines ---
	
	-- 10a) Defeat line 1 (mob or played was killed)
	initiator_name = string.match(line, "^.* a vaincu (.*)%.$");

	if (initiator_name ~= nil) then

	-- Update
		return 9, trim_articles(initiator_name);
	end

	-- 10b) Defeat line 2 (mob died)
	initiator_name = string.match(line, "^(.*) meurt%.$");

	if (initiator_name ~= nil) then
		
		-- Update
		return 9, trim_articles(initiator_name);
	end

	-- 10c) Defeat line 3 (a player was killed or died)
	initiator_name = string.match(line, "^(.*) a péri%.$");

	if (initiator_name ~= nil) then
		
		-- Update
		return 9, trim_articles(initiator_name);
	end

	-- 10d) Defeat line 4 (you were killed)
	match = string.match(line, "^.* a réussi à vous mettre hors de combat%.$");

	if (match ~= nil) then
		
		-- Update
		return 9, trim_articles(player.name);
	end

	-- 10e) Defeat line 5 (you died)
	match = string.match(line, "^Un incident vous a réduit à l'impuissance%.$");

	if (match ~= nil) then
		
		-- Update
		return 9, trim_articles(player.name);
	end	
	-- 10f) Defeat line 6 (you killed a mob)
	local initiatorName = string.match(line,"^Votre coup puissant a vaincu (.*)%.$");
	
	if (initiatorName ~= nil) then
		
		-- Update
		return 9,trim_articles(initiatorName);
	end
	
	-- 11) Revive lines --
	
	-- 11a) Revive line 1 (player revived)
	local initiatorName = string.match(line,"^(.*) revient à la vie%.$");
	
	if (initiatorName ~= nil) then
	  
		-- Update
	  return 10,trim_articles(initiatorName);
	end
	
	-- 11b) Revive line 2 (player succumbed)
	local initiatorName = string.match(line,"^(.*) a succombé à ses blessures%.$");
	
	if (initiatorName ~= nil) then
	  
		-- Update
	  return 10,trim_articles(initiatorName);
	end
	
	-- 11c) Revive line 3 (you were revived)
	local match = string.match(line,"^Vous revenez à la vie%.$");
	
	if (match ~= nil) then
	  initiatorName = player.name;
	  
		-- Update
	  return 10,initiatorName;
	end
	
	-- 11d) Revive line 4 (you succumbed)
	local match = string.match(line,"^You succumb to your wounds%.$");
	
	if (match ~= nil) then
	  initiatorName = player.name;
	  
		-- Update
	  return 10,initiatorName;
	end
	
	-- if we reach here, we were unable to parse the line
	--  (note there is very little that isn't parsed)
end

function ChatTypeToString(chattype)

	if chattype == Turbine.ChatType.PlayerCombat then
		return "PlayerCombat"
	elseif chattype == Turbine.ChatType.EnemyCombat then
		return "EnemyCombat"
	elseif chattype == Turbine.ChatType.Say then
		return "Say"
	elseif chattype == Turbine.ChatType.Advancement then
		return "Advancement"
	elseif chattype == Turbine.ChatType.Emote then
		return "Emote"
	elseif chattype == Turbine.ChatType.FellowLoot then
		return "FellowLoot"
	elseif chattype == Turbine.ChatType.Kinship then
		return "Kinship"
	elseif chattype == Turbine.ChatType.Raid then
		return "Raid"
	elseif chattype == Turbine.ChatType.LFF then
		return "LFF"
	elseif chattype == Turbine.ChatType.World then
		return "World"
	elseif chattype == Turbine.ChatType.SelfLoot then
		return "SelfLoot"
	elseif chattype == Turbine.ChatType.FellowLoot then
		return "FellowLoot"
	elseif chattype == Turbine.ChatType.OOC then
		return "OOC"
	elseif chattype == Turbine.ChatType.Regional then
		return "Regional"
	elseif chattype == Turbine.ChatType.Raid then
		return "Raid"
	elseif chattype == Turbine.ChatType.Trade then
		return "Trade"
	elseif chattype == Turbine.ChatType.Tribe then
		return "Tribe"
	elseif chattype == Turbine.ChatType.Officer then
		return "Officer"
	elseif chattype == Turbine.ChatType.UserChat1 then
		return "UserChat1"
	elseif chattype == Turbine.ChatType.UserChat2 then
		return "UserChat2"
	elseif chattype == Turbine.ChatType.UserChat3 then
		return "UserChat3"
	elseif chattype == Turbine.ChatType.UserChat4 then
		return "UserChat4"
	else
		return chattype
	end
	

end

function CreateBaseTrigger()

    local data = {}

    data["token"] = ""
    data["description"] = ""
    data["target_name"] = ""
    data["timer"] = 0
    data["icon"] = nil
    data["id"] = Turbine.Engine.GetGameTime()
    data["text"] = nil
    data["threshold"] = 3
    data["popup_multi"] = 2
    data["flashing"] = true
    data["loop"] = false
    data["reset"] = false
    data["unique"] = false
    data["sort_index"] = 0
    data["type"] = 0
    data["protected"] = false
	data["use_regex"] = false

    return data

end

function addtowindow_NewEffectSelf(w_data)

    local index = table.getn(w_data["list_effects_self"]) + 1

    w_data["list_effects_self"][index] = CreateBaseTrigger()
    w_data["list_effects_self"][index]["sort_index"] = w_data["next_index"]
    w_data["list_effects_self"][index]["type"] = EFFECTS_SELF
    w_data["next_index"] = w_data["next_index"] + 1

    SaveSettings()
    return w_data["list_effects_self"][index]

end

function addtowindow_NewEffectGroup(w_data)

    local index = table.getn(w_data["list_effects_group"]) + 1

    w_data["list_effects_group"][index] = CreateBaseTrigger()
    w_data["list_effects_group"][index]["sort_index"] = w_data["next_index"]
    w_data["list_effects_group"][index]["type"] = EFFECTS_GROUP
    w_data["next_index"] = w_data["next_index"] + 1

    SaveSettings()
    return w_data["list_effects_group"][index]

end

function addtowindow_NewEffectTarget(w_data)

    local index = table.getn(w_data["list_effects_target"]) + 1

    w_data["list_effects_target"][index] = CreateBaseTrigger()
    w_data["list_effects_target"][index]["sort_index"] = w_data["next_index"]
    w_data["list_effects_target"][index]["type"] = EFFECTS_TARGET
    w_data["next_index"] = w_data["next_index"] + 1

    SaveSettings()
    return w_data["list_effects_target"][index]

end

function addtowindow_NewChat(w_data)

    local index = table.getn(w_data["list_chat"]) + 1

    w_data["list_chat"][index] = CreateBaseTrigger()
    w_data["list_chat"][index]["sort_index"] = w_data["next_index"]
    w_data["list_chat"][index]["type"] = CHAT
    w_data["next_index"] = w_data["next_index"] + 1

    SaveSettings()
    return w_data["list_chat"][index]

end

function addtowindow_NewSkill(w_data)

    local index = table.getn(w_data["list_skill"]) + 1

    w_data["list_skill"][index] = CreateBaseTrigger()
    w_data["list_skill"][index]["sort_index"] = w_data["next_index"]
    w_data["list_skill"][index]["type"] = SKILL
    w_data["next_index"] = w_data["next_index"] + 1

    SaveSettings()
    return w_data["list_skill"][index]

end

function addtowindow_NewTrigger_End(w_data)

    local index = table.getn(w_data["list_triggers_end"]) + 1

    w_data["list_triggers_end"][index] = CreateBaseTrigger()
    w_data["list_triggers_end"][index]["sort_index"] = w_data["next_index"]
    w_data["list_triggers_end"][index]["type"] = TRIGGER_END
    w_data["next_index"] = w_data["next_index"] + 1

    SaveSettings()
    return w_data["list_triggers_end"][index]

end


function addtowindow_NewTrigger_Start(w_data)

    local index = table.getn(w_data["list_triggers_start"]) + 1

    w_data["list_triggers_start"][index] = CreateBaseTrigger()
    w_data["list_triggers_start"][index]["sort_index"] = w_data["next_index"]
    w_data["list_triggers_start"][index]["type"] = TRIGGER_START
    w_data["next_index"] = w_data["next_index"] + 1

    SaveSettings()
    return w_data["list_triggers_start"][index]

end


function window_GetIndex(window)
    for i,v in ipairs(windowdata) do

        if v["load"] then

            if windowdata[i] == window.data then
                return i
            end
        end
	end
	Turbine.Shell.WriteLine(window.data["name"])
    return 0
end


function addtowindow_NewTrigger_Threshold(w_data)

    local index = table.getn(w_data["list_triggers_threshold"]) + 1

    w_data["list_triggers_threshold"][index] = CreateBaseTrigger()
    w_data["list_triggers_threshold"][index]["sort_index"] = w_data["next_index"]
    w_data["list_triggers_threshold"][index]["type"] = TRIGGER_THRESHOLD
    w_data["next_index"] = w_data["next_index"] + 1

    SaveSettings()
    return w_data["list_triggers_threshold"][index]

end

function DuplicateWindow(index)

	local index2 = AddNewWindow()

	windowdata[index2]["name"] = 						windowdata[index]["name"]
	windowdata[index2]["type"] = 						windowdata[index]["type"]
	windowdata[index2]["load"] =						windowdata[index]["load"] 
	windowdata[index2]["width"] = 						windowdata[index]["width"] 
	windowdata[index2]["height"] = 						windowdata[index]["height"]
	windowdata[index2]["left"] = 						windowdata[index]["left"] 
	windowdata[index2]["top"] = 						windowdata[index]["top"] 
	windowdata[index2]["frame"] = 						windowdata[index]["frame"] 
	windowdata[index2]["spacing"] = 					windowdata[index]["spacing"]
	windowdata[index2]["next_index"] =					windowdata[index]["next_index"]
	windowdata[index2]["frame_color"]["R"] =			windowdata[index]["frame_color"]["R"]
	windowdata[index2]["frame_color"]["G"] =			windowdata[index]["frame_color"]["G"]
	windowdata[index2]["frame_color"]["B"] =			windowdata[index]["frame_color"]["B"]
	windowdata[index2]["back_color"]["R"] =				windowdata[index]["back_color"]["R"]
	windowdata[index2]["back_color"]["G"] =				windowdata[index]["back_color"]["G"]
	windowdata[index2]["back_color"]["B"] =				windowdata[index]["back_color"]["B"]
	windowdata[index2]["bar_color"]["R"] = 				windowdata[index]["bar_color"]["R"]
	windowdata[index2]["bar_color"]["G"] = 				windowdata[index]["bar_color"]["G"]
	windowdata[index2]["bar_color"]["B"] = 				windowdata[index]["bar_color"]["B"]
	windowdata[index2]["font"] =						windowdata[index]["font"]
	windowdata[index2]["opacity"] = 					windowdata[index]["opacity"] 
	windowdata[index2]["ascending"] = 					windowdata[index]["ascending"]
	windowdata[index2]["orientation_horizontal"] =		windowdata[index]["orientation_horizontal"]

	local coin = 10000
	for i,v in ipairs(windowdata[index]["list_effects_self"]) do
		local t = addtowindow_NewEffectSelf(windowdata[index2])

		t["token"] = v["token"]
		t["description"] = v["description"]
		t["target_name"] = v["target_name"]
		t["timer"] = v["timer"]
		t["icon"] = v["icon"]
		t["text"] = v["text"]
		t["threshold"] = v["threshold"]
		t["popup_multi"] = v["popup_multi"]
		t["flashing"] = v["flashing"]
		t["loop"] = v["loop"]
		t["reset"] = v["reset"]
		t["unique"] = v["unique"]
		t["protected"] = v["protected"]
		t["use_regex"] = v["use_regex"]
		t["id"] = v["id"]-coin

		coin = coin+10000
	end

	for i,v in ipairs(windowdata[index]["list_effects_group"]) do
		local t = addtowindow_NewEffectGroup(windowdata[index2])

		t["token"] = v["token"]
		t["description"] = v["description"]
		t["target_name"] = v["target_name"]
		t["timer"] = v["timer"]
		t["icon"] = v["icon"]
		t["text"] = v["text"]
		t["threshold"] = v["threshold"]
		t["popup_multi"] = v["popup_multi"]
		t["flashing"] = v["flashing"]
		t["loop"] = v["loop"]
		t["reset"] = v["reset"]
		t["unique"] = v["unique"]
		t["protected"] = v["protected"]
		t["use_regex"] = v["use_regex"]
		t["id"] = v["id"]-coin

		coin = coin+10000

	end

	for i,v in ipairs(windowdata[index]["list_chat"]) do
		local t = addtowindow_NewChat(windowdata[index2])

		t["token"] = v["token"]
		t["description"] = v["description"]
		t["target_name"] = v["target_name"]
		t["timer"] = v["timer"]
		t["icon"] = v["icon"]
		t["text"] = v["text"]
		t["threshold"] = v["threshold"]
		t["popup_multi"] = v["popup_multi"]
		t["flashing"] = v["flashing"]
		t["loop"] = v["loop"]
		t["reset"] = v["reset"]
		t["unique"] = v["unique"]
		t["protected"] = v["protected"]
		t["use_regex"] = v["use_regex"]
		t["id"] = v["id"]-coin

		coin = coin+10000

	end

	for i,v in ipairs(windowdata[index]["list_skill"]) do
		local t = addtowindow_NewSkill(windowdata[index2])

		t["token"] = v["token"]
		t["description"] = v["description"]
		t["target_name"] = v["target_name"]
		t["timer"] = v["timer"]
		t["icon"] = v["icon"]
		t["text"] = v["text"]
		t["threshold"] = v["threshold"]
		t["popup_multi"] = v["popup_multi"]
		t["flashing"] = v["flashing"]
		t["loop"] = v["loop"]
		t["reset"] = v["reset"]
		t["unique"] = v["unique"]
		t["protected"] = v["protected"]
		t["use_regex"] = v["use_regex"]
		t["id"] = v["id"]-coin

		coin = coin+10000

	end

	for i,v in ipairs(windowdata[index]["list_effects_target"]) do
		local t = addtowindow_NewEffectTarget(windowdata[index2])

		t["token"] = v["token"]
		t["description"] = v["description"]
		t["target_name"] = v["target_name"]
		t["timer"] = v["timer"]
		t["icon"] = v["icon"]
		t["text"] = v["text"]
		t["threshold"] = v["threshold"]
		t["popup_multi"] = v["popup_multi"]
		t["flashing"] = v["flashing"]
		t["loop"] = v["loop"]
		t["reset"] = v["reset"]
		t["unique"] = v["unique"]
		t["protected"] = v["protected"]
		t["use_regex"] = v["use_regex"]
		t["id"] = v["id"]-coin

		coin = coin+10000

	end

	for i,v in ipairs(windowdata[index]["list_triggers_end"]) do
		local t = addtowindow_NewTrigger_End(windowdata[index2])

		t["token"] = v["token"]
		t["description"] = v["description"]
		t["target_name"] = v["target_name"]
		t["timer"] = v["timer"]
		t["icon"] = v["icon"]
		t["text"] = v["text"]
		t["threshold"] = v["threshold"]
		t["popup_multi"] = v["popup_multi"]
		t["flashing"] = v["flashing"]
		t["loop"] = v["loop"]
		t["reset"] = v["reset"]
		t["unique"] = v["unique"]
		t["protected"] = v["protected"]
		t["use_regex"] = v["use_regex"]
		t["id"] = v["id"]-coin

		coin = coin+10000

	end

	for i,v in ipairs(windowdata[index]["list_triggers_start"]) do
		local t = addtowindow_NewTrigger_Start(windowdata[index2])

		t["token"] = v["token"]
		t["description"] = v["description"]
		t["target_name"] = v["target_name"]
		t["timer"] = v["timer"]
		t["icon"] = v["icon"]
		t["text"] = v["text"]
		t["threshold"] = v["threshold"]
		t["popup_multi"] = v["popup_multi"]
		t["flashing"] = v["flashing"]
		t["loop"] = v["loop"]
		t["reset"] = v["reset"]
		t["unique"] = v["unique"]
		t["protected"] = v["protected"]
		t["use_regex"] = v["use_regex"]
		t["id"] = v["id"]-coin

		coin = coin+10000

	end

	for i,v in ipairs(windowdata[index]["list_triggers_threshold"]) do
		local t = addtowindow_NewTrigger_Threshold(windowdata[index2])

		t["token"] = v["token"]
		t["description"] = v["description"]
		t["target_name"] = v["target_name"]
		t["timer"] = v["timer"]
		t["icon"] = v["icon"]
		t["text"] = v["text"]
		t["threshold"] = v["threshold"]
		t["popup_multi"] = v["popup_multi"]
		t["flashing"] = v["flashing"]
		t["loop"] = v["loop"]
		t["reset"] = v["reset"]
		t["unique"] = v["unique"]
		t["protected"] = v["protected"]
		t["use_regex"] = v["use_regex"]
		t["id"] = v["id"]-coin

		coin = coin+10000

	end

	SaveSettings()

end

function SortWindows()

	for i=1, table.getn(windowdata), 1 do 

		local index = i
		for j=i, table.getn(windowdata), 1 do 

			if SortStrings(windowdata[index]["name"],windowdata[j]["name"]) then
				index = j
			end

		end

		local temp = windowdata[i]
		windowdata[i] = windowdata[index]
		windowdata[index] = temp

	end

end


function AddNewWindow()

    local index = table.getn(windowdata)+1

    windowdata[index] = {}
    windowdata[index]["name"] = "New Window "..windowdata["next_id"]
    windowdata[index]["type"] = "Bar_ListBox"
    windowdata[index]["id"] = windowdata["next_id"]
    windowdata[index]["load"] = true
    windowdata[index]["group"] = nil
    windowdata[index]["width"] = 150
    windowdata[index]["height"] = 20
    windowdata[index]["left"] = 100
    windowdata[index]["top"] = 100
    windowdata[index]["frame"] = 1
    windowdata[index]["spacing"] = 2
    windowdata[index]["next_index"] = 1
    windowdata[index]["frame_color"] = {}
    windowdata[index]["frame_color"]["R"] = 0
    windowdata[index]["frame_color"]["G"] = 0
    windowdata[index]["frame_color"]["B"] = 0
    windowdata[index]["back_color"] = {}
    windowdata[index]["back_color"]["R"] = 1
    windowdata[index]["back_color"]["G"] = 1
    windowdata[index]["back_color"]["B"] = 1
    windowdata[index]["bar_color"] = {}
    windowdata[index]["bar_color"]["R"] = 0
    windowdata[index]["bar_color"]["G"] = 0
    windowdata[index]["bar_color"]["B"] = 0
    windowdata[index]["font"] = 23
    windowdata[index]["opacity"] = 0.5
    windowdata[index]["ascending"] = false
    windowdata[index]["orientation_horizontal"] = false
    windowdata[index]["list_effects_self"] = {}
    windowdata[index]["list_effects_group"] = {}
    windowdata[index]["list_chat"] = {}
    windowdata[index]["list_skill"] = {}
    windowdata[index]["list_effects_target"] = {}
    windowdata[index]["list_triggers_end"] = {}
    windowdata[index]["list_triggers_start"] = {}
    windowdata[index]["list_triggers_threshold"] = {}
    --windowdata[index]["all_self_debuffs"] = true
    --windowdata[index]["all_self_debuffs_data"] = CreateBaseTrigger()
    --windowdata[index]["all_self_debuffs_blacklist"] = {}
    --windowdata[index]["all_self_buffs"] = false

    windowdata["next_id"] = windowdata["next_id"] +1

    window_Create(index)

    SaveSettings()

    return index

end

function RemoveWindow(index)

    if windowdata[index] ~= nil then

        window_Close(index)

        local count = table.getn(windowdata)

        if count == 1 then

            windowdata[index] = nil


        elseif count > 1  then

            windowdata[index] = nil
            windowdata[index] = windowdata[count]
            windowdata[count] = nil

        else
            Turbine.Shell.WriteLine("Remove Window Error!")
        end

        SaveSettings()

    end
    
end

function SortStrings(a, b)--return a > b

	local len = math.min(string.len(a), string.len(b))

	for i=1, len, 1 do
		local a1 = string.sub(a, i, i)
		local b1 = string.sub(b, i, i)
		if a1 > b1 then
			return true
		elseif b1 > a1 then
			return false
		end
	end
	if string.len(a) == len then
		return false
	else
		return true
	end

end


function SortGroups()

	for i=1, table.getn(windowdata), 1 do 

		local index = i
		for j=i, table.getn(windowdata["window_groups"]), 1 do 

			if SortStrings(windowdata["window_groups"][index]["name"],windowdata["window_groups"][j]["name"]) then
				index = j
			end

		end

		local temp = windowdata["window_groups"][i]
		windowdata["window_groups"][i] = windowdata["window_groups"][index]
		windowdata["window_groups"][index] = temp

	end

end


function AddNewGroup(name)

    local index = table.getn(windowdata["window_groups"]) + 1

    windowdata["window_groups"][index] = {}
    windowdata["window_groups"][index]["load"] = true
    windowdata["window_groups"][index]["name"] = name
    windowdata["window_groups"][index]["collapsed"] = false
    windowdata["window_groups"][index]["id"] = windowdata["next_group_id"]

    windowdata["next_group_id"] = windowdata["next_group_id"] + 1

	SortGroups()

    SaveSettings()

    return index

end

function RemoveGroup(index)

	local count = table.getn(windowdata["window_groups"])
	local id = windowdata["window_groups"][index]["id"]

    if count == 1 then
        windowdata["window_groups"][index] = nil
    elseif count > 1 then
        windowdata["window_groups"][index] = nil
        windowdata["window_groups"][index] = windowdata["window_groups"][count]
        windowdata["window_groups"][count] = nil
    else
        return
    end
  
    for i=1, table.getn(windowdata), 1 do
        if windowdata[i]["group"] == id then
            windowdata[i]["group"] = nil
        end
    end

    SaveSettings()

end

function trigger_GetIndex(trigger, list)
    for i=1, table.getn(list), 1 do
        if trigger == list[i] then
            return i
        end
    end
end


function trigger_Remove(trigger, window)

    local list
    local type = trigger["type"]

    if type == EFFECTS_SELF then
        list = window["list_effects_self"]
    elseif type == EFFECTS_GROUP then
        list = window["list_effects_group"]
    elseif type == EFFECTS_TARGET then
        list = window["list_effects_target"]
    elseif type == CHAT then
        list = window["list_chat"]
    elseif type == SKILL then
        list = window["list_skill"]
    elseif type == TRIGGER_END then
        list = window["list_triggers_end"]
    elseif type == TRIGGER_START then
        list = window["list_triggers_start"]
    elseif type == TRIGGER_THRESHOLD then
        list = window["list_triggers_threshold"]
    end
    
    local count = table.getn(list)

    if count == 1 then 
        list[1] = nil
    elseif count == 0 then

    else
        local index = trigger_GetIndex(trigger, list)
        list[index] = list[count]
        list[count] = nil
    end

    SaveSettings()

end

function reload_after_groupchanges_Changed(bool)

    windowdata["reload_after_groupchanges"] = bool
    c_icon:Reload_After_Groupchanges()
    if settings_window ~= nil then
        --settings_window:Reload_After_Groupchanges()
    end

            
    SaveSettings()

end

---------------------------------------------------------------------
--== window-functions ===--
---------------------------------------------------------------------

function group_Close()

end

function move_selection_Changed(item)
	move_selection = item
	if move_window ~= nil then
		move_window:MoveSelectionChanged()
	end
end

function move_group(x_diff, y_diff)

	if move_selection ~= nil then

		for i,windata in ipairs(windowdata) do 

			if (windata["load"] == true) and windata["group"] == move_selection.data["id"] then
	
				windows[i]:GroupMove(x_diff, y_diff)
	
			end
	
		end

	end

end

function move_selection_Moved()
	if move_window ~= nil then
		move_window:MoveSelectionMoved()
	end
	if move_selection.data["group"] ~= nil then
		for i=1, table.getn(windowdata["window_groups"]), 1 do
			if windowdata["window_groups"][i]["load"] then
				if windowdata["window_groups"][i]["id"] == move_selection.data["group"] then
					groups[i]:ChildMoved()
				end
			end
		end
	end
end


function show_groups_changed(bool)

	show_groups = bool

	for i,data in ipairs(windowdata["window_groups"]) do 
		if data["load"] then
			groups[i]:Show_Changed()
		end
	end

end

function window_MovingChanged()

    moving = not(moving)

    for i,windata in ipairs(windowdata) do 

        if (windata["load"]) then

            windows[i]:MovingChanged()

        end

	end
	
	if moving then

		move_selection = nil
		move_window = UI.Settings.MoveWindow()

		for i,data in ipairs(windowdata["window_groups"]) do 
			if data["load"] then
				groups[i] = UI.Windows.Group(data)
			end
		end

	else

		move_window:Close()

		for i,data in ipairs(windowdata["window_groups"]) do 
			if data["load"] then
				groups[i]:Close()
				groups[i] = nil
			end
		end

	end
	OptionsReload()

end


function window_Create(index)

    if(windowdata[index]["type"] == "Bar_ListBox") then
        windows[index] = UI.Windows.Bar_ListBox(windowdata[index])

    elseif( windowdata[index]["type"] == "Bar_Window") then
        windows[index] = UI.Windows.Bar_Window(windowdata[index])

    elseif( windowdata[index]["type"] == "Icon_ListBox") then
        windows[index] = UI.Windows.Icon_ListBox(windowdata[index])

    elseif( windowdata[index]["type"] == "Icon_Window") then
        windows[index] = UI.Windows.Icon_Window(windowdata[index])

    else

        Turbine.Shell.WriteLine("SaveFile corrupted!")

    end

    SaveSettings()

end

function window_Close(index)

    if windowdata[index]["load"] then
		if windows[index] ~= nil then
        	windows[index]:Close()
		end
    end

end

function window_CreateAll()

    for i=1, table.getn(windowdata), 1 do

        if(windowdata[i]["load"] == true) then
    
           window_Create(i)
    
        end
    
    end
    SaveSettings()

end

function window_DataChanged()

    for i=1, table.getn(windowdata), 1 do

        if(windowdata[i]["load"] == true) then
            windows[i]:Reload()
        end

    end

end

---------------------------------------------------------------------
--== settings-functions ===--
---------------------------------------------------------------------

function settings_Open()

    if settings_window == nil then
        settings_window = UI.Settings.Options.Window(windowdata)
    else
        settings_window:Close()
		settings_window = nil
    end
	OptionsReload()

end

function settings_Close( reload )

	if settings_window ~= nil then
		settings_window:Close()
		settings_window = nil
	end

    if reload then

        ReloadPlugin("Gibberish reload reason: \"options saved\"")

	else

		OptionsReload()

    end

end

function TrackGroupChanged()
    if windowdata["track_effects_group"] then
        Trigger.AddGroupEffectEvents()
    end
    c_icon:TrackGroupChanged()
end

function TrackTargetChanged()
    if windowdata["track_effects_target"] then
        Trigger.AddEventTargetEffects()
    end
    c_icon:TrackTargetChanged()
	if collectors_window ~= nil then
		collectors_window:TrackTargetChanged()
	end
end

---------------------------------------------------------------------
--== collection-functions ===--
---------------------------------------------------------------------

function check_activ_Effects()

    if collect_effects then
        Trigger.check_activ_Effects_forCollection_Self()


        if windowdata["track_effects_group"] then
            Trigger.check_activ_Effects_forCollection_Group()
        end
    end

    if collect_target_effects then
        if windowdata["track_effects_target"] then
            Trigger.check_activ_Effects_forCollection_Target()
        end
    end

end

function collection_Open()

    if collectors_window == nil then
        collectors_window = UI.Settings.Collection.Window()
    else
        collectors_window:Close()
    end
	OptionsReload()

end

function collection_Close()

    collectors_window = nil
	OptionsReload()

end

function  EffectCollectionChanged(index)
    if collectors_window ~= nil then
        collectors_window:EffectCollectionChanged(index)
    end

    if settings_window ~= nil then
        settings_window:EffectCollectionChanged(index)
    end
end

function  ChatCollectionChanged(index)
    if collectors_window ~= nil then
        collectors_window:ChatCollectionChanged(index)
    end

    if settings_window ~= nil then
        settings_window:ChatCollectionChanged(index)
    end
end

function SetTarget_CollectEffects(value)
    collect_target_effects = value
end

function SetTarget_OnlyBuffs(value)
    effects_target_only_buffs = value
end

function SetEffects_Collect(value)
    collect_effects = value
end

function SetChat_Collect(value)
    collect_chat = value
end

function SetEffect_OnlyDebuffs(value)
    effects_only_debuffs = value
end

function SetChat_OnlySay(value)
    chat_only_say = value
end

---------------------------------------------------------------------
--== chat comand ===--
---------------------------------------------------------------------

command = Turbine.ShellCommand()

function command:Execute( _, str )
    if str == nil or string.len( str ) == 0 then
        Turbine.Shell.WriteLine( "Missing Argument (reset | reload).")
        return
    end

    local list  = { }
    local index = 1

    for word in str:gmatch( "%w+" ) do
        list[ index ] = word
        index = index + 1
    end

    local cmd = string.lower( list[ 1 ] )

    if cmd == "options" then

        settings_Open()

    elseif cmd == "print" then

        local string = "Gibberish Print:\n"

        for k,v in ipairs(windowdata) do
            if v["load"] then
                string = string.."["..windowdata[k]["name"].."] <load>\n"
                for i,t in ipairs(windows[k].controls) do
                    string = string.." > ["..t.data["text"].."] "..t.data["token"].."\n"
                end
            else
                string = string.."["..windowdata[k]["name"].."] <not load>\n"
            end
        end

        Turbine.Shell.WriteLine(string)

    elseif cmd == "reload" then

        ReloadPlugin(nil)

    elseif cmd == "move" then

        moving = not(moving)
        MovingChanged()
       
    elseif cmd == "reset" then

        if table.getn(list) == 2 then

            for i,w_data in ipairs(windowdata) do
                if string.lower(w_data["name"]) == string.lower(list[2]) then

                    windows[i]:ResetAll()

                    return
                end
            end

        else
        
            for i,w_data in ipairs(windowdata) do
                if w_data["load"] then
                    windows[i]:ResetAll()
                end
            end

        end

    else
        Turbine.Shell.WriteLine( "Invalid command: " .. cmd )
    end

end

Turbine.Shell.AddCommand( "gibberish", command )

---------------------------------------------------------------------
--== load in ===--
---------------------------------------------------------------------

function LoadTriggerFromReloadFile(list)

	--local list = start_running_triggerlist

    if locale ~= "en" then
        list = convertFromEuro(list);
    end

    local gt = Turbine.Engine.GetGameTime()

    for i,windata in ipairs(windowdata) do 

        if list[i] ~= nil then
			
            for j, trig in ipairs(list[i]) do
                if (trig["start_time"] + trig["duration"]) > gt and trig["data"] ~= nil then
                    windows[i]:CheckForAdd(windows[i], trig["key"], trig["start_time"], trig["duration"], trig["data"]["icon"], trig["text"], trig["data"], true)
                end
            end
        end
    end
end



if Turbine.PluginData.Load(Turbine.DataScope.Account, "gibberish_account_file" .. fileSuffix) ~= nil then

	windowdata = Turbine.PluginData.Load(Turbine.DataScope.Account, "gibberish_account_file" .. fileSuffix)
	if locale ~= "en" then
		windowdata = convertFromEuro(windowdata);
	end

    if Turbine.PluginData.Load(Turbine.DataScope.Character, "gibberish_char_file" .. fileSuffix) ~= nil then

        local chardata = Turbine.PluginData.Load(Turbine.DataScope.Character, "gibberish_char_file" .. fileSuffix)
		if locale ~= "en" then
			chardata = convertFromEuro(chardata);
		end
	
        windowdata["logo"] = chardata["logo"]
        windowdata["settings"] = chardata["settings"]
        windowdata["track_effects_group"] = chardata["track_effects_group"]
        windowdata["track_effects_target"] = chardata["track_effects_target"]

        if chardata["reload_after_groupchanges"] == nil then
            windowdata["reload_after_groupchanges"] = true
        else
            windowdata["reload_after_groupchanges"] = chardata["reload_after_groupchanges"]
        end

        if windowdata["window_groups"] == nil then
            windowdata["window_groups"] = {}
            chardata["window_groups"] = {}
        end

        if chardata["window_groups"] == nil then
            chardata["window_groups"] = {}
        end


        if windowdata["next_group_id"] == nil then
            windowdata["next_group_id"] = 1
        end

        for key, value in ipairs(chardata["window_groups"]) do
            if value["id"] == nil then
                value["id"] = key
            end
        end

        for key, value in ipairs(windowdata["window_groups"]) do
            if value["id"] == nil then
                value["id"] = windowdata["next_group_id"]
                windowdata["next_group_id"] = windowdata["next_group_id"] + 1
            end

			local bool = false
            for k, v in ipairs(chardata["window_groups"]) do

                if value["id"] == v["id"] then
                    value["load"] = v["load"] 
					value["collapsed"] =  v["collapsed"] 
					bool = true
                end
			end
			if bool == false then
				value["load"] = false
			end
        end

        for i,w_data in ipairs(windowdata) do

			for key, value in ipairs(LIST_NAMES) do
                if w_data[value] == nil then
                    w_data[value] = {}
                else
                    for k,v in ipairs(w_data[value]) do
                        if v["id"] == nil then
                            v["id"] = i..k..Turbine.Engine.GetGameTime()
                        end
                    end
                end
            end



            local bool = false

            for j,c_data in ipairs(chardata) do

                if w_data["id"] == c_data["id"] then
                    w_data["load"] = c_data["load"]
                    w_data["width"] =  c_data["width"]
                    w_data["height"] =  c_data["height"]
                    w_data["left"] = c_data["left"]
                    w_data["top"] =  c_data["top"]

                    bool = true
                end

            end

            if bool == false then
                w_data["load"] = false
            end

        end

    else

        for i,w_data in ipairs(windowdata) do

            w_data["load"] = false
            w_data["left"] = 100
            w_data["top"] = 200

        end
        
    end

else

    windowdata["next_id"] = 1
    windowdata["track_effects_group"] = true
    windowdata["track_effects_target"] = false
    windowdata["reload_after_groupchanges"] = true

    windowdata["next_group_id"] = 1
    windowdata["window_groups"] = {}

    windowdata["logo"] = {}
    windowdata["logo"]["left"] = 300
    windowdata["logo"]["top"] = 400
    windowdata["logo"]["locked"] = false

    windowdata["settings"] = {}
    windowdata["settings"]["left"] = 400
    windowdata["settings"]["top"] = 200
    windowdata["settings"]["width"] = 550
	windowdata["settings"]["height"] = 400
    windowdata["settings"]["show_tooltips"] = true

    windowdata["collection"] = {}
    windowdata["collection"]["left"] = 400
    windowdata["collection"]["top"] = 200
    windowdata["collection"]["width"] = 200
    windowdata["collection"]["height"] = 330


end



function LoadOptionsFromReloadFile(list)

		if list["show_groups"] == false then
		show_groups = false
	else
		show_groups = true
	end
	if list["move_windows"] == true then
		window_MovingChanged()
	end

	if list["options_open"] == true then
		settings_Open()
		if list["options_windowselection"] ~= false then
			if windowdata[list["options_windowselection"]] ~= nil then
				settings_window:WindowSelectionChanged(list["options_windowselection"])
			end
		end
	end

	if list["collection_open"] == true then
		collection_Open()
	end

end



import "Souru.Gibberish.Trigger"


Trigger.window_CreateAll()
Trigger.checkAllSkills()
Trigger.AddSelfEffectEvents()
if windowdata["track_effects_target"] then
    Trigger.AddEventTargetEffects()
end
if windowdata["track_effects_group"] then
    Trigger.AddGroupEffectEvents()
end

if Turbine.PluginData.Load(Turbine.DataScope.Account, "gibberish_library" .. fileSuffix) ~= nil then
	library = Turbine.PluginData.Load(Turbine.DataScope.Account, "gibberish_library" .. fileSuffix)
	if locale ~= "en" then
        library = convertFromEuro(library);
    end
end

--start_running_triggerlist = nil

if Turbine.PluginData.Load(Turbine.DataScope.Character, "gibberish_running_triggerlist" .. fileSuffix) ~= nil then
	--start_running_triggerlist = 
    LoadTriggerFromReloadFile(Turbine.PluginData.Load(Turbine.DataScope.Character,  "gibberish_running_triggerlist" .. fileSuffix))
end

-- delay = Turbine.UI.Control()
-- delayTimer = Turbine.Engine.GetGameTime()+1
-- delay.Update = function()

-- 	if delayTimer <= Turbine.Engine.GetGameTime() then
-- 		delay:SetWantsUpdates(false)
-- 		LoadTriggerFromReloadFile()
-- 	end

-- end
-- delay:SetWantsUpdates(true)

if Turbine.PluginData.Load(Turbine.DataScope.Character, "gibberish_optionwindow_state" .. fileSuffix) ~= nil then
    LoadOptionsFromReloadFile(Turbine.PluginData.Load(Turbine.DataScope.Character,  "gibberish_optionwindow_state" .. fileSuffix))
end

c_icon = UI.Settings.ControlIcon(windowdata["logo"])