
ControlIcon = class( Turbine.UI.Window )

function ControlIcon:Constructor( data )
    Turbine.UI.Window.Constructor( self )

    self.data = data
    
    self:Draw()
    self:Move(self.data["locked"])

end



---------------------------------------------------------------------
--== public ===--
---------------------------------------------------------------------


---------------------------------------------------------------------
--== private ===--
---------------------------------------------------------------------

function ControlIcon:Draw()

    self:SetSize(32,32)
    self:SetPosition(self.data["left"], self.data["top"])
    self:SetOpacity(0.5)

    self.icon = Turbine.UI.Control()
    self.icon:SetParent(self)
    self.icon:SetSize(32,32)
    self.icon:SetMouseVisible(false)

    self.icon:SetBackground("Souru/Gibberish/Resources/logo.tga")

    self.track_target_label = Turbine.UI.Label()
    self.track_target_label:SetParent(self)
    self.track_target_label:SetSize(32,28)
    self.track_target_label:SetMouseVisible(false)
    self.track_target_label:SetPosition(-2,2)
    self.track_target_label:SetTextAlignment(Turbine.UI.ContentAlignment.BottomLeft)
    self.track_target_label:SetFont(Turbine.UI.Lotro.Font.Verdana18)
    self.track_target_label:SetText(" T")
    self.track_target_label:SetVisible(windowdata["track_effects_target"])
    self.track_target_label:SetForeColor(Turbine.UI.Color.Yellow)
    self.track_target_label:SetFontStyle(Turbine.UI.FontStyle.Outline)


    self.track_group_label = Turbine.UI.Label()
    self.track_group_label:SetParent(self)
    self.track_group_label:SetSize(34,30)
    self.track_group_label:SetMouseVisible(false)
    self.track_group_label:SetPosition(0,0)
    self.track_group_label:SetTextAlignment(Turbine.UI.ContentAlignment.TopRight)
    self.track_group_label:SetFont(Turbine.UI.Lotro.Font.Verdana18)
    self.track_group_label:SetText("G ")
    self.track_group_label:SetVisible(windowdata["track_effects_group"])
    self.track_group_label:SetForeColor(Turbine.UI.Color(0, 0.47, 1))
    self.track_group_label:SetFontStyle(Turbine.UI.FontStyle.Outline)

    

    self:SetMouseVisible(true)
    self:SetVisible(true)

    self.MouseClick = function(sender, args)

        if args.Button == Turbine.UI.MouseButton.Right then
            self.menu:ShowMenu()
        end

    end

    self.MouseEnter = function(sender, args)
        self:SetOpacity(1)
    end

    self.MouseLeave = function(sender, args)
       self:SetOpacity(0.5)
    end

    self:DrawMenu()

end

function ControlIcon:TrackTargetChanged()
    self.track_target_label:SetVisible(windowdata["track_effects_target"])
end

function ControlIcon:TrackGroupChanged()

    self.track_group_label:SetVisible(windowdata["track_effects_group"])
end

function ControlIcon:DrawMenu()

    self.item_options = Turbine.UI.MenuItem(  Utils.L[locale]["context_menu"]["options"], true )
    self.item_reset = Turbine.UI.MenuItem( Utils.L[locale]["context_menu"]["reset"], true )
    self.item_reload = Turbine.UI.MenuItem( Utils.L[locale]["context_menu"]["reload"], true)
    self.item_collectors = Turbine.UI.MenuItem(Utils.L[locale]["context_menu"]["collection"], true)
    self.item_move = Turbine.UI.MenuItem(  Utils.L[locale]["context_menu"]["move_windows"], true, moving )
    self.item_lock = Turbine.UI.MenuItem(  Utils.L[locale]["context_menu"]["move_icon"], true, not(self.data["locked"]) )
    self.item_reload_after_groupchanges = Turbine.UI.MenuItem(  Utils.L[locale]["context_menu"]["group_changed"], true, windowdata["reload_after_groupchanges"] )

    self.menu = Turbine.UI.ContextMenu()
    self.items = self.menu:GetItems()

    self.items:Add( self.item_reset )
    self.items:Add( self.item_options )
    self.items:Add( self.item_reload)
    self.items:Add( self.item_collectors )
    self.items:Add( self.item_move )
    self.items:Add( self.item_lock )
    self.items:Add( self.item_reload_after_groupchanges )

    self.item_reset.Click = function(sender, args)

        for i,w_data in ipairs(windowdata) do
            if w_data["load"] then
                windows[i]:ResetAll()
            end
        end

    end

    self.item_reload_after_groupchanges.Click = function(sender, args)

        reload_after_groupchanges_Changed(not(self.item_reload_after_groupchanges:IsChecked()))

    end

    self.item_options.Click = function(sender, args)

        settings_Open()

    end

    self.item_reload.Click = function(sender, args)

        ReloadPlugin(nil)

    end

    self.item_collectors.Click = function(sender, args)

        collection_Open()

    end


    self.item_lock.Click = function(sender, args)

        self.data["locked"] = not(self.data["locked"])
        self:Move(self.data["locked"])
        self.item_lock:SetChecked(not(self.data["locked"]))
        
    end

    self.item_move.Click = function(sender, args)

        window_MovingChanged()
        self.item_move:SetChecked(moving)

    end

end

function ControlIcon:Reload_After_Groupchanges()

    self.item_reload_after_groupchanges:SetChecked(windowdata["reload_after_groupchanges"])

end

function ControlIcon:Move(locked)

    if locked then
      
        self.MouseDown = function( sender, args )

        end
        
        self.MouseMove = function( sender, args )

        end
        
        self.MouseUp = function( sender, args )

        end

    else

        self.MouseDown = function( sender, args )
            if args.Button == Turbine.UI.MouseButton.Left then
                self.dragging = true	
                self.dragStartX = args.X
                self.dragStartY = args.Y
            end
        end
        
        self.MouseMove = function( sender, args )
            if self.dragging then
                local x,y = self:GetPosition()	
                x = x + ( args.X - self.dragStartX )
                y = y + ( args.Y - self.dragStartY )	
                self:SetPosition( x, y )	
            end
        end
        
        self.MouseUp = function( sender, args )
            if args.Button == Turbine.UI.MouseButton.Left then
                self.dragging = false
                self.data["left"],self.data["top"] = self:GetPosition()
                    
                SaveSettings()
            end
        end

    end

end