C_WIDTH = 200
C_HEIGHT = 120

ConfirmationWindow = class( Turbine.UI.Window )

function ConfirmationWindow:Constructor(parent, type, data, delete_children, id)
    Turbine.UI.Window.Constructor( self )

    self.parent = parent
    self.data = data
    self.type = type
    self.delete_children = delete_children
    self.id = id

    self:Draw()
    self:SetParent(parent)
    self:Resize()

    if type == 0 then
        self.text:SetText(Utils.L[locale]["are_you_sure"]..self.data["token"].."</rgb>")
    elseif type == 1 then
        self.text:SetText(Utils.L[locale]["are_you_sure"]..windowdata[self.data]["name"].."</rgb>")
    elseif type == 2 then
        self.text:SetText(Utils.L[locale]["are_you_sure"]..windowdata["window_groups"][self.data]["name"].."</rgb>")
    end

    self:SetVisible(true)
    self.frame:SetVisible(true)

end

---------------------------------------------------------------------
--== public ===--
---------------------------------------------------------------------


---------------------------------------------------------------------
--== private ===--
---------------------------------------------------------------------

function ConfirmationWindow:CloseSelf()

    self:SetVisible(false)
    self.frame:SetVisible(false)
    self.frame:Close()
    self:Close()

end

function ConfirmationWindow:YesClicked()

    if self.type == 0 then
        self:DeleteTrigger()
    elseif self.type == 1 then
        self:DeleteWindow()
    elseif self.type == 2 then
        self:DeleteGroup()
    end

    self:CloseSelf()

end

function ConfirmationWindow:NoClicked()
    self:CloseSelf()
end

function ConfirmationWindow:DeleteGroup()

    if self.delete_children then
        local list = {}
        local count = 1
        for i=1, table.getn(windowdata), 1 do
            if windowdata[i]["group"] == self.id then
                list[count] = i
                count = count +1

            end
        end
        for i=table.getn(list), 1, -1 do
            RemoveWindow(list[i])
        end
    end

    RemoveGroup(self.data)
    self.parent:FillWindows()

end

function ConfirmationWindow:DeleteWindow()
    RemoveWindow(self.data)
    self.parent:FillWindows()

    self.parent.selected_window_index = nil
    self.parent:FillTrigger()
    self.parent:FillOptions()
    self.parent:EnableOptions(false)
end

function ConfirmationWindow:DeleteTrigger()
    trigger_Remove(self.data, windowdata[self.parent.selected_window_index])
    self.parent:FillTrigger()
    SaveSettings()
end

function ConfirmationWindow:Resize()

    self:SetSize(self:GetParent():GetWidth(),self:GetParent():GetHeight()-25)
    self.frame:SetPosition((self:GetWidth()/2)-150, (self:GetHeight()/2)-75)

end

function ConfirmationWindow:Draw()

    self:SetZOrder(10)
    self:SetBackColor(Turbine.UI.Color.Black)
    self:SetTop(25)
    self:SetOpacity(0.6)


    self.frame = Turbine.UI.Window()
    self.frame:SetParent(self)
    self.frame:SetBackColor(Turbine.UI.Color.White)
    self.frame:SetSize(C_WIDTH,C_HEIGHT)

    self.header = Turbine.UI.Label()
    self.header:SetParent(self.frame)
    self.header:SetSize(C_WIDTH, 25)
    self.header:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
    self.header:SetFont(Turbine.UI.Lotro.Font.Verdana14)
    self.header:SetForeColor(Turbine.UI.Color.Black)
    self.header:SetText(Utils.L[locale]["header"]["delete_confirmation"])

    self.header:SetMouseVisible(false)
    self.header:SetZOrder(5)
    
    self.background = Turbine.UI.Control()
    self.background:SetParent(self.frame)
    self.background:SetPosition(4,25)
    self.background:SetBackColor(Turbine.UI.Color.Black)
    self.background:SetSize(C_WIDTH-8,C_HEIGHT-25-4)
    self.background:SetMouseVisible(false)
    self.background:SetZOrder(2)

    self.text = Turbine.UI.Label()
    self.text:SetParent(self.frame)
    self.text:SetPosition(5, 25)
    self.text:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
    self.text:SetFont(Turbine.UI.Lotro.Font.Verdana14)
    self.text:SetForeColor(Turbine.UI.Color.White)
    self.text:SetSize(C_WIDTH-10,C_HEIGHT-54)
    self.text:SetMouseVisible(false)
    self.text:SetMarkupEnabled(true)
    self.text:SetZOrder(5)

    self.yes_button = Turbine.UI.Lotro.Button()
    self.yes_button:SetParent(self.frame)
    self.yes_button:SetPosition(9, self.frame:GetHeight() - 9 - 20)
    self.yes_button:SetSize(75,20)
    self.yes_button:SetText(Utils.L[locale]["button"]["yes"])
    self.yes_button:SetVisible(true)
    self.yes_button:SetZOrder(6)
    self.yes_button.Click = function()

        self:YesClicked()

    end

    self.no_button = Turbine.UI.Lotro.Button()
    self.no_button:SetParent(self.frame)
    self.no_button:SetPosition(self.frame:GetWidth()-9-75, self.frame:GetHeight() - 9 - 20)
    self.no_button:SetSize(75,20)
    self.no_button:SetText(Utils.L[locale]["button"]["no"])
    self.no_button:SetZOrder(6)
    self.no_button:SetVisible(true)
    self.no_button.Click = function()

        self:NoClicked()

    end

    self:Focus()

end