EXPORTWINDOW_WIDTH = 300
EXPORTWINDOW_HEIGHT = 400

ExportWindow = class( Turbine.UI.Window )

function ExportWindow:Constructor(parent)
    Turbine.UI.Window.Constructor( self )

    self.parent = parent

    self.screen_width , self.screen_height = Turbine.UI.Display:GetSize()

    self:SetSize(EXPORTWINDOW_WIDTH + 5,  EXPORTWINDOW_HEIGHT + 30)
    self:SetPosition(300,400)
    self:SetMouseVisible(false)

    self.back = UI.Settings.BaseWindow(EXPORTWINDOW_WIDTH, EXPORTWINDOW_HEIGHT,  0,  0, Turbine.UI.Color.White, Turbine.UI.Color.Black, heading, self)
    self.back:ChangeVisiblility(true)
    self.back:SetZOrder(2)

    self.foreground = Turbine.UI.Window()
    self.foreground:SetParent(self)
    self.foreground:SetPosition(5, 25)
    self.foreground:SetSize(EXPORTWINDOW_WIDTH +5, EXPORTWINDOW_HEIGHT +5)
    self.foreground:SetVisible(true)
    self.foreground:SetZOrder(6)

    self.textbox = Turbine.UI.Lotro.TextBox()
    self.textbox:SetParent(self.foreground)
	self.textbox:SetSize(EXPORTWINDOW_WIDTH, EXPORTWINDOW_HEIGHT)
    self.textbox:SetFont(TB_FONT)
    self.textbox:SetFontStyle(Turbine.UI.FontStyle.None)
    self.textbox:SetTextAlignment(Turbine.UI.ContentAlignment.TopLeft)

    self.v_scroll = Turbine.UI.Lotro.ScrollBar();
    self.v_scroll:SetParent(self.textbox);
    self.v_scroll:SetOrientation(Turbine.UI.Orientation.Vertical);
    self.v_scroll:SetSize(12,self.textbox:GetHeight());
    self.v_scroll:SetLeft((self.textbox:GetWidth() - 12));
    self.v_scroll:SetVisible(false);
    self.v_scroll:SetZOrder(200001);
    self.textbox:SetVerticalScrollBar(self.v_scroll);
    
    self.accept_button = Turbine.UI.Lotro.Button()
    self.accept_button:SetParent(self.foreground)
    self.accept_button:SetPosition(self.foreground:GetWidth() - 100, self.foreground:GetHeight() - 35)
    self.accept_button:SetSize(80,20)
    self.accept_button:SetText(Utils.L[locale]["button"]["import"] )
    self.accept_button:SetVisible(true)

    self.accept_button.Click = function(sender, args)
        self:Determine_Type()
    end

end


---------------------------------------------------------------------
--== public ===--
---------------------------------------------------------------------

function ExportWindow:ShowExport_Group(name, datalist)
    self.data = datalist
    self.back:SetText(Utils.L[locale]["header"]["export"])
    self.accept_button:SetVisible(false)
    self.textbox:SetReadOnly(true)

    local text = self:ConvertGroupToString(name)
    for j=1, table.getn(datalist), 1 do
        text = text..self:ConvertWindowToString(datalist[j])
        for i,list_name in pairs(LIST_NAMES) do 
            for key,t_data in ipairs(datalist[j][list_name]) do 
                text = text..self:ConvertTriggerToString(t_data)
            end
        end
    end
    self.textbox:SetText(text)
    
    self:SetVisible(true)
end

function ExportWindow:ShowExport_Window(data)
    self.data = data
    self.back:SetText(Utils.L[locale]["header"]["export"])
    self.accept_button:SetVisible(false)
    self.textbox:SetReadOnly(true)

    local text = self:ConvertWindowToString(data, nil, nil)
    for i,list_name in pairs(LIST_NAMES) do 
        for key,t_data in ipairs(data[list_name]) do 
            text = text..self:ConvertTriggerToString(t_data)
        end
    end
    self.textbox:SetText(text)
    
    self:SetVisible(true)
end

function ExportWindow:ShowExport_Trigger(data)
    self.data = data
    self.back:SetText(Utils.L[locale]["header"]["export"])
    self.accept_button:SetVisible(false)
    self.textbox:SetReadOnly(true)

    local text = ""

    text = text..self:ConvertTriggerToString(data)


    self.textbox:SetText(text)
    
    self:SetVisible(true)
end

function ExportWindow:ShowImport()
    self.data = nil
    self.back:SetText(Utils.L[locale]["header"]["import"])
    self.accept_button:SetVisible(true)
    self.textbox:SetReadOnly(false)

    self:SetVisible(true)
end


---------------------------------------------------------------------
--== private ===--
---------------------------------------------------------------------

function ExportWindow:ConvertBoolean(value)

    if value == "true" then
        return true
    elseif value == "false" then
        return false
    end

    return nil

end

function ExportWindow:Determine_Type()

    local text = self.textbox:GetText()
    text = string.gsub(text, "\n", "")

    if string.find(text, "=== group ===") then
        local g_name, data_list = self:InterpreteGroup(text)
        self.parent:AddGroupFromImport(g_name, data_list)
    elseif string.find(text, "=== window ===") then
        local w_data = self:InterpreteWindow(text)
        self.parent:AddWindowFromImport(w_data)
    elseif string.find(text, "=== trigger ===") then
        local t_data = self:InterpreteTrigger(text)
        self.parent:AddTriggerFromImport(t_data)
    end

end

function ExportWindow:InterpreteGroup(text)

    local start_index1, end_index1  = string.find(text, "=== group ===name=")
    local start_index2, end_index2  = string.find(text, ";=== group end ===")
    local group_name = string.sub(text, end_index1 + 1, start_index2 - 1)
    text = split(text, ";=== group end ===")[2]

    local window_splitted_text  = split(text, "=== window ===")

    local w_data_list = {}
    for i,window_string in pairs(window_splitted_text) do

        if window_string ~= "" and window_string ~= nil then

            w_data_list[table.getn(w_data_list) + 1] = self:InterpreteWindow("=== window ==="..window_string)

        end

    end

    return group_name, w_data_list

end

function ExportWindow:InterpreteWindow(text)

    local w_data = {}

    for key, value in ipairs(LIST_NAMES) do
        w_data[value] = {}
    end

    local start_index1, end_index1  = string.find(text, "=== window ===")
    local start_index2, end_index2  = string.find(text, ";=== window end ===")
    local w_info_string = string.sub(text, end_index1 + 1, start_index2 - 1)
    text = split(text,  ";=== window end ===")[2]

    local zeilen = split(w_info_string, ";")

    for i,zeile in pairs(zeilen) do

        if zeile ~= "" then

            local split_attributes = split(zeile, "~=")
            local key = split_attributes[1]
            local attribute = split_attributes[2]

            if key == "next_index" then
                w_data[key] = tonumber(attribute)

            elseif key == "opacity" then
                w_data[key] = tonumber(attribute)

            elseif key == "opacity2" then
                w_data[key] = tonumber(attribute)

            elseif key == "left" then

                local value = tonumber(attribute)
                if value <= 1 then 
                    w_data[key] = value*self.screen_width
                else
                    w_data[key] = 200
                end

            elseif key == "top" then

                local value = tonumber(attribute)
                if value <= 1 then 
                    w_data[key] = value*self.screen_height
                else
                    w_data[key] = 200
                end

            elseif key == "width" then
                w_data[key] = tonumber(attribute)

            elseif key == "height" then
                w_data[key] = tonumber(attribute)

            elseif key == "id" then
                w_data[key] = tonumber(attribute)

            elseif key == "font" then
                w_data[key] = tonumber(attribute)

            elseif key == "frame" then
                w_data[key] = tonumber(attribute)

            elseif key == "spacing" then
                w_data[key] = tonumber(attribute)

            elseif key == "group" then
                w_data[key] = tonumber(attribute)

            elseif key == "ascending" then
                w_data[key] = self:ConvertBoolean(attribute)

            elseif key == "load" then
                w_data[key] = self:ConvertBoolean(attribute)

            else
                w_data[key] = attribute
            end

        end

    end

    w_data["frame_color"] = {}
    w_data["frame_color"]["R"] = 0
    w_data["frame_color"]["G"] = 0
    w_data["frame_color"]["B"] = 0
    w_data["back_color"] = {}
    w_data["back_color"]["R"] = 1
    w_data["back_color"]["G"] = 1
    w_data["back_color"]["B"] = 1
    w_data["bar_color"] = {}
    w_data["bar_color"]["R"] = 0
    w_data["bar_color"]["G"] = 0
    w_data["bar_color"]["B"] = 0

    local trigger_list = self:InterpreteTrigger(text)
 
    for i=1, table.getn(trigger_list), 1 do
        w_data[ LIST_NAMES[ trigger_list[i]["type"] ] ][table.getn(w_data[ LIST_NAMES[ trigger_list[i]["type"] ] ]) +1] = trigger_list[i]
    end

    return w_data

end

function ExportWindow:InterpreteTrigger(text)

    local t_list = {}
    
    local splitted_text  = split(text, "=== trigger ===")

    for i,trigger_string in pairs(splitted_text) do
        
        if trigger_string ~= "" then
            local t_data = {}

            
            local zeilen = split(trigger_string, ";")

            for i,zeile in pairs(zeilen) do

                if zeile ~= "" then

                    local split_attributes = split(zeile, "~=")
                    local key = split_attributes[1]
                    local attribute = split_attributes[2]

                    if key == "timer" then
                        t_data[key] = tonumber(attribute)

                    elseif key == "threshold" then
                        t_data[key] = tonumber(attribute)

                    elseif key == "icon" then
                        t_data[key] = tonumber(attribute)

                    elseif key == "type" then
                        t_data[key] = tonumber(attribute)

                    elseif key == "loop" then
                        t_data[key] = self:ConvertBoolean(attribute)

                    elseif key == "flashing" then
                        t_data[key] = self:ConvertBoolean(attribute)

                    elseif key == "unique" then
                        t_data[key] = self:ConvertBoolean(attribute)

                    elseif key == "reset" then
                        t_data[key] = self:ConvertBoolean(attribute)

                    elseif key == "use_regex" then
                        t_data[key] = self:ConvertBoolean(attribute)

                    elseif key == "hide_text" then
                        t_data[key] = self:ConvertBoolean(attribute)

                    elseif key == "protected" then
                        t_data[key] = self:ConvertBoolean(attribute)

                    elseif key == "sort_index" then
                        t_data[key] = tonumber(attribute)

                    else
                        t_data[key] = attribute
                    end

                end

            end

            
            t_list[table.getn(t_list) + 1] = t_data

        end
        
    end

    return t_list

end

function ExportWindow:Closed()
    self.parent:CloseExportWindow()
end

function ExportWindow:ConvertTriggerToString(t_data)

    text = "\n=== trigger ===\n"
    for key,value in pairs(t_data) do 
        text = text..key.."~="..tostring(value)..";\n"
    end

    return text

end

function ExportWindow:ConvertWindowToString(w_data)

    text = "\n=== window ===\n"
    for key,value in pairs(w_data) do 
        if type(value) ~= "table" then

            if key == "left" then

                text = text..key.."~="..tostring(value/self.screen_width)..";\n"

            elseif key == "top" then

                text = text..key.."~="..tostring(value/self.screen_height)..";\n"

            elseif key == "group" then

            else
                text = text..key.."~="..tostring(value)..";\n"
            end
        end
    end
    text = text.."=== window end ===\n"
    return text

end

function ExportWindow:ConvertGroupToString(name)

    text = "\n=== group ===\n"
    text = text.."name="..name..";\n"
    text = text.."=== group end ===\n"
    return text

end