C_WIDTH = 200
C_HEIGHT = 120

GroupNameWindow = class( Turbine.UI.Window )

function GroupNameWindow:Constructor(parent, index)
    Turbine.UI.Window.Constructor( self )

    self.parent = parent
    self.index = index

    self:Draw()
    self:SetParent(parent)
    self:Resize()

    if index ~= nil then
        self.textBox:SetText(windowdata["window_groups"][index]["name"])
    end

    self:SetVisible(true)
    self.frame:SetVisible(true)

end

---------------------------------------------------------------------
--== public ===--
---------------------------------------------------------------------


---------------------------------------------------------------------
--== private ===--
---------------------------------------------------------------------

function GroupNameWindow:CloseSelf()

    self:SetVisible(false)
    self.frame:SetVisible(false)
    self.frame:Close()
    self:Close()

end

function GroupNameWindow:AcceptClicked()

    if self.index == nil then
        AddNewGroup(self.textBox:GetText())
    else
        windowdata["window_groups"][self.index]["name"] = self.textBox:GetText()
        SortGroups()
        SaveSettings()
    end    

    self.parent:FillGroupContextMenu()
    self.parent:FillWindows()
    self:CloseSelf()

end

function GroupNameWindow:CancelClicked()
    self:CloseSelf()
end


function GroupNameWindow:Resize()

    self:SetSize(self:GetParent():GetWidth(),self:GetParent():GetHeight()-25)
    self.frame:SetPosition((self:GetWidth()/2)-150, (self:GetHeight()/2)-75)

end

function GroupNameWindow:Draw()

    self:SetZOrder(10)
    self:SetBackColor(Turbine.UI.Color.Black)
    self:SetTop(25)
    self:SetOpacity(0.6)


    self.frame = Turbine.UI.Window()
    self.frame:SetParent(self)
    self.frame:SetBackColor(Turbine.UI.Color.White)
    self.frame:SetSize(C_WIDTH,C_HEIGHT)

    self.header = Turbine.UI.Label()
    self.header:SetParent(self.frame)
    self.header:SetSize(C_WIDTH, 25)
    self.header:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
    self.header:SetFont(Turbine.UI.Lotro.Font.Verdana14)
    self.header:SetForeColor(Turbine.UI.Color.Black)
    self.header:SetText(Utils.L[locale]["header"]["group_name"])

    self.header:SetMouseVisible(false)
    self.header:SetZOrder(5)
    
    self.background = Turbine.UI.Control()
    self.background:SetParent(self.frame)
    self.background:SetPosition(4,25)
    self.background:SetBackColor(Turbine.UI.Color.Black)
    self.background:SetSize(C_WIDTH-8,C_HEIGHT-25-4)
    self.background:SetMouseVisible(false)
    self.background:SetZOrder(2)

    self.text = Turbine.UI.Label()
    self.text:SetParent(self.frame)
    self.text:SetPosition(5, 30)
    self.text:SetTextAlignment(Turbine.UI.ContentAlignment.TopCenter)
    self.text:SetFont(Turbine.UI.Lotro.Font.Verdana14)
    self.text:SetForeColor(Turbine.UI.Color.White)
    self.text:SetSize(C_WIDTH-10,C_HEIGHT-54)
    self.text:SetMouseVisible(false)
    self.text:SetMarkupEnabled(true)
    self.text:SetZOrder(5)
    self.text:SetText(Utils.L[locale]["group_name_text"])

    self.textBox = Turbine.UI.Lotro.TextBox()
	self.textBox:SetParent(self.frame)
	self.textBox:SetSize(C_WIDTH-40,25)
	self.textBox:SetPosition(20,53)
	self.textBox:SetFont(Turbine.UI.Lotro.Font.Verdana14)
    self.textBox:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft)
    self.textBox:SetText("New Group")
    self.textBox:SetZOrder(5)


    self.accept_button = Turbine.UI.Lotro.Button()
    self.accept_button:SetParent(self.frame)
    self.accept_button:SetPosition(9, self.frame:GetHeight() - 9 - 20)
    self.accept_button:SetSize(75,20)
    self.accept_button:SetText(Utils.L[locale]["button"]["accept"])
    self.accept_button:SetVisible(true)
    self.accept_button:SetZOrder(6)
    self.accept_button.Click = function()

        self:AcceptClicked()

    end

    self.cancel_button = Turbine.UI.Lotro.Button()
    self.cancel_button:SetParent(self.frame)
    self.cancel_button:SetPosition(self.frame:GetWidth()-9-75, self.frame:GetHeight() - 9 - 20)
    self.cancel_button:SetSize(75,20)
    self.cancel_button:SetText(Utils.L[locale]["button"]["cancel"])
    self.cancel_button:SetZOrder(6)
    self.cancel_button:SetVisible(true)
    self.cancel_button.Click = function()

        self:CancelClicked()

    end

    self:Focus()
    self.frame:Focus()
    self.textBox:SelectAll()
    self.textBox:Focus()

end