
Bar_ListBox = class( Turbine.UI.Window )

function Bar_ListBox:Constructor(data)
	Turbine.UI.Window.Constructor( self )

    self.data = data  
    self.controls = {}

    if type(self.data["ascending"]) ~= "boolean" then
		self.data["ascending"] = false
	end

	if type(self.data["left"]) ~= "number" then
		self.data["left"] = 100
	end

	if type(self.data["top"]) ~= "number" then
		self.data["top"] = 100
	end

	if type(self.data["width"]) ~= "number" then
		self.data["width"] = 150
	end

	if type(self.data["height"]) ~= "number" then
		self.data["height"] = 20
	end

	if type(self.data["spacing"]) ~= "number" then
		self.data["spacing"] = 2
	end

	if type(self.data["frame"]) ~= "number" then
		self.data["frame"] = 1
	end

    self:Draw()

end


---------------------------------------------------------------------
--== public ===--
---------------------------------------------------------------------

function Bar_ListBox:GetRunningTriggerList()

    local list = {}

    for key, control in ipairs(self.controls) do
        list[key] = control:GetTriggerInfo()
    end

    return list

end

function Bar_ListBox:AddItem(item)	

    self.list:AddItem(item)
    self:Sort()
    self:Resize()

end

function Bar_ListBox:Reload()--DataChanged

end



function Bar_ListBox:FindKey(key)

    for k,v in ipairs(self.controls) do
        if key == v.key then
            return k
            
        end
    end
    return nil

end

function Bar_ListBox:CheckForAdd(parent, key, start_time, duration, icon, text, private_data, running)

    local index = self:FindKey(key)

    if index == nil then

        local i = table.getn(self.controls) + 1

        self.controls[i] = UI.Windows.Items.Bar_L(parent, key, start_time, duration, icon, text, private_data, true)
        self:AddItem(self.controls[i])
        Trigger.check_for_waiting_trigger_start(self.controls[i].data["id"]..self.controls[i].key)
    else
        self.controls[index]:Reset(start_time, duration, icon, text)
        Trigger.check_for_waiting_trigger_start(self.controls[index].data["id"]..self.controls[index].key)
    end

end

function Bar_ListBox:ResetAll()

    for k,v in ipairs(self.controls) do
        if v.data["reset"] then
            self:DeleteItem(v)
            v:Delete()
        end
    end

end

function Bar_ListBox:CheckForRemove(key)

    for k,v in ipairs(self.controls) do
        if not(v.data["protected"]) then
            if  string.find(v.key, key) then
                self:DeleteItem(v)
                break
            end
        end
    end

end

function Bar_ListBox:DeleteItem(item)	

    local index = self:FindIndex(item)
    if index ~= nil then

        self:RemoveItem(item)
        local length = table.getn(self.controls)
        self.controls[index] = self.controls[length]
        self.controls[length] = nil
        item:Delete()
    end

end

function Bar_ListBox:RemoveItem(item)	

    if self.list:ContainsItem(item) then
        self.list:RemoveItem(item)
		self:Resize()
    end

end

function Bar_ListBox:Reset()

    for k,v in ipairs(self.controls) do

        if v.data["reset"] then
            self:DeleteItem(v)
            v:Delete()
        end

    end

    self:Sort()
    self:Resize()

end

function Bar_ListBox:MovingChanged()
    self.move_window:SetMouseVisible(moving)
    self.move_window:SetVisible(moving)

    if moving then
		self:SetZOrder(10)
	else
		self:SetZOrder(nil)
	end
end

function Bar_ListBox:Sort()
    self.list:Sort(function(elem1,elem2) if elem1.endtime<elem2.endtime then return true end end)
end

---------------------------------------------------------------------
--== private ===--
---------------------------------------------------------------------


function Bar_ListBox:FindIndex(item)

    for k,v in ipairs(self.controls) do
        if item == v then
            return k
            
        end
    end
    return nil

end

function Bar_ListBox:Resize()

    local itemCount = self.list:GetItemCount() + 1
    local height = self.data["height"]+self.data["spacing"]
    local full_height = itemCount*(self.data["height"]+self.data["spacing"])
    
    self:SetHeight(full_height)
    self.list:SetHeight(full_height)
    self.move_window:SetHeight(height)
    self.move_label:SetHeight(height-2)

end

function Bar_ListBox:Set_Size(width, height)

    local itemCount = self.list:GetItemCount() + 1
    local full_height = itemCount * height

    self:SetSize(width, full_height)
    self.list:SetSize(width, full_height)
    self.move_window:SetSize(width+self.data["frame"], height)
    self.move_label:SetSize(width-2+self.data["frame"], height-2)

end

function Bar_ListBox:GroupMove(x_diff, y_diff)

    self.data["left"] = self.data["left"] + x_diff
    self.data["top"] = self.data["top"] + y_diff

    self:SetPosition(self.data["left"], self.data["top"])

end

function Bar_ListBox:Draw()

    local width = self.data["width"] + self.data["height"] + (3*self.data["frame"])
    local height = (self.data["height"]+self.data["spacing"])

    self:SetPosition(self.data["left"], self.data["top"])
	self:SetMouseVisible(false)

    self.move_window = Turbine.UI.Window()
    self.move_window:SetParent(self)
    self.move_window:SetMouseVisible(moving)
    self.move_window:SetVisible(moving)
    self.move_window:SetTop(self.data["spacing"])
    self.move_window:SetBackColor(Turbine.UI.Color(0.3,0.3,0.3))
    self.move_window:SetOpacity(0.8)

    self.move_label = Turbine.UI.Label()
	self.move_label:SetParent(self.move_window)
	self.move_label:SetPosition(1,1)
	self.move_label:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
	self.move_label:SetFont(Turbine.UI.Lotro.Font.Verdana12)
    self.move_label:SetFontStyle(Turbine.UI.FontStyle.Outline)
    self.move_label:SetText(self.data["name"])
    self.move_label:SetMouseVisible(false)
    self.move_label:SetBackColor(Turbine.UI.Color.Black)
    self.move_label:SetForeColor(Turbine.UI.Color.White)

	self.list = Turbine.UI.ListBox()
	self.list:SetParent(self)
	self.list:SetMouseVisible(false)
    self.list:SetTop(0)
    self.list:SetReverseFill(self.data["ascending"])
    self.list:SetFlippedLayout(self.data["ascending"])
    

    
    self.move_window.MouseDown = function( sender, args )
		if args.Button == Turbine.UI.MouseButton.Left then
			self.dragging = true	
			self.dragStartX = args.X
            self.dragStartY = args.Y
            move_selection_Changed(self)
		end
	end
	
	self.move_window.MouseMove = function( sender, args )
		if self.dragging then
			local x,y = self:GetPosition()	
			x = x + ( args.X - self.dragStartX )
            y = y + ( args.Y - self.dragStartY )	
            self.data["left"],self.data["top"] = self:GetPosition()
            self:SetPosition( x, y )	
            move_selection_Moved()
		end
	end
	
	self.move_window.MouseUp = function( sender, args )
		if args.Button == Turbine.UI.MouseButton.Left then
			self.dragging = false
			self.data["left"],self.data["top"] = self:GetPosition()
				
			SaveSettings()
		end
    end
    
    self:Set_Size(width, height)
    self:SetVisible(true)

end

function Bar_ListBox:LeftChanged(left)

    self.data["left"] = left
	self:SetLeft(left)
    move_selection_Moved()
    SaveSettings()

end

function Bar_ListBox:TopChanged(top)

    self.data["top"] = top
	self:SetTop(top)
    move_selection_Moved()
    SaveSettings()

end

function Bar_ListBox:WidthChanged(width)

    self.data["width"] = width
	local width = self.data["width"] + self.data["height"] + (2*self.data["frame"])
    self.move_window:SetWidth(width+self.data["frame"])
    self.move_label:SetWidth(width+self.data["frame"]-2)
	self:SetSize(width+self.data["frame"])
    SaveSettings()

    for key, control in ipairs(self.controls) do
        control:UpdateSize()
    end

end


function Bar_ListBox:HeightChanged(height)

    local width = self.data["width"] + height + (2*self.data["frame"])
    self.data["height"] = height
    self.move_window:SetSize(width+self.data["frame"], height+(2*self.data["frame"]))
    self.move_label:SetSize(width+self.data["frame"]-2, height+(2*self.data["frame"])-2)
	self:SetSize(width+self.data["frame"], height+(2*self.data["frame"]))
    SaveSettings()

    for key, control in ipairs(self.controls) do
        control:UpdateSize()
    end

end


function Bar_ListBox:GetLeftForGroup()
    return self:GetLeft()
end

function Bar_ListBox:GetTopForGroup()
    return self:GetTop()
end

function Bar_ListBox:GetRightForGroup()
    return self:GetLeft() + self:GetWidth()
end

function Bar_ListBox:GetBottomForGroup()
    return self:GetTop() + self:GetHeight()
end