
Icon_Window = class( Turbine.UI.Window )

function Icon_Window:Constructor(data)
	Turbine.UI.Window.Constructor( self )

    self.data = data  
	self.controls = {}
	
	if type(self.data["ascending"]) ~= "boolean" then
		self.data["ascending"] = false
	end

	if type(self.data["left"]) ~= "number" then
		self.data["left"] = 100
	end

	if type(self.data["top"]) ~= "number" then
		self.data["top"] = 100
	end

	if type(self.data["width"]) ~= "number" then
		self.data["width"] = 150
	end

	if type(self.data["height"]) ~= "number" then
		self.data["height"] = 20
	end

	if type(self.data["spacing"]) ~= "number" then
		self.data["spacing"] = 2
	end

	if type(self.data["frame"]) ~= "number" then
		self.data["frame"] = 1
	end

	self:Draw()
	self:Fill()

end

---------------------------------------------------------------------
--== public ===--
---------------------------------------------------------------------

function Icon_Window:GetRunningTriggerList()
	local list = {}

	for key, control in ipairs(self.controls) do
        list[key] = control:GetTriggerInfo()
    end

    return list
end

function Icon_Window:SettingsUpdated()

end

function Icon_Window:CheckForRemove(key)
	
    for k,v in ipairs(self.controls) do
        if not(v.data["protected"]) then
            if  string.find(key, v.key) ~= nil then
                v:Reset(Turbine.Engine.GetGameTime(), 0)
                break
            end
        end
    end

end

function Icon_Window:AddItem(item)

end

function Icon_Window:DeleteItem(item)

end

function Icon_Window:RemoveItem(item)

end

function Icon_Window:CheckForAdd(parent, key, start_time, duration, icon, text, private_data, running)

	for k,v in ipairs(self.controls) do

		if string.find(key, v.key) and icon == v.data["icon"] then
			v:Reset(start_time, duration, icon, text)
			Trigger.check_for_waiting_trigger_start(v.data["id"]..v.key)
        end
		
	end

end

function Icon_Window:Close()
	self:SetVisible(false)
end

function Icon_Window:Reset()

	for k,v in ipairs(self.controls) do

        if v.data["reset"] then
            self:Reset(Turbine.Engine.GetGameTime(), 0)
        end

    end

    self:Sort()
    self:Resize()

end

function Icon_Window:MovingChanged()
	self.move_window:SetMouseVisible(moving)
	self.move_window:SetVisible(moving)
	
	if moving then
		self:SetZOrder(10)
	else
		self:SetZOrder(1)
	end
end

function Icon_Window:Sort()
    self.list:Sort(function(elem1,elem2) if elem1.data["sort_index"]<elem2.data["sort_index"] then return true end end)
end

function Icon_Window:Reload()--DataChanged

	self:Fill()

end

---------------------------------------------------------------------
--== private ===--
---------------------------------------------------------------------

function Icon_Window:Fill()

	self.list:ClearItems()
	for k,v in ipairs(self.controls) do
		v:Delete()
	end
	self.controls = {}

	local skill_list = lp:GetTrainedSkills()

	local selfIndex = window_GetIndex(self)


	for k,v in ipairs(LIST_NAMES) do
		
		for i,item in ipairs(self.data[v]) do 
			if v == "list_skill" then
				for j=1, skill_list:GetCount(), 1 do
					local s_name = skill_list:GetItem(j):GetSkillInfo():GetName()
					
					if s_name ==self.data[v][i]["token"] then
						local index = table.getn(self.controls) + 1
						self.controls[index] = UI.Windows.Items.Icon(SettingsOverwrite(item, self, string.gsub(item["token"], "-", ""), 0, item["timer"], nil, item["token"], false))
						self.list:AddItem(self.controls[index])
						break
					end
				end
			else
				local index = table.getn(self.controls) + 1
				self.controls[index] = UI.Windows.Items.Icon(SettingsOverwrite(item, self, string.gsub(item["token"], "-", ""), 0, item["timer"], nil, item["token"], false))
				self.list:AddItem(self.controls[index])
			end
		end
	end

	self:Sort()
	self:Resize()

end

function Icon_Window:ResetAll()

    for k,v in ipairs(self.controls) do
		if v.data["reset"] then
            v:Passiv()
        end
    end

end

function Icon_Window:Resize()

    local itemCount = 2--table.getn(self.controls) + 1
    local height = self.data["height"]+self.data["spacing"]
    local full_height = itemCount*(self.data["height"]+self.data["spacing"])

    self:SetHeight(full_height)
    self.list:SetHeight(full_height)
    self.move_window:SetHeight(height-self.data["spacing"])
    self.move_label:SetHeight(height-2-self.data["spacing"])

end

function Icon_Window:Set_Size(width, height)

    local itemCount = 2--table.getn(self.controls) + 1
    local full_height = itemCount * height

    self:SetSize(width, full_height)
    self.list:SetSize(width, height)
    self.move_window:SetSize(width, height-self.data["spacing"])
    self.move_label:SetSize(width-2, height-2- self.data["spacing"])

end


function Icon_Window:Draw()

    local width = self.data["width"]
    local height = (self.data["height"]+self.data["spacing"])

    self:SetPosition(self.data["left"], self.data["top"])
	self:SetMouseVisible(false)
	self:SetZOrder(1)
    self:SetVisible(true)

	self.list = Turbine.UI.ListBox()
	self.list:SetParent(self)
	self.list:SetMouseVisible(false)
	self.list:SetTop(0)
    self.list:SetReverseFill(self.data["ascending"])
    --self.list:SetFlippedLayout(self.data["ascending"])

    if self.data["orientation_horizontal"] then
        self.list:SetMaxItemsPerLine(1)
    end
    
    self.move_window = Turbine.UI.Window()
    self.move_window:SetParent(self)
    self.move_window:SetMouseVisible(moving)
    self.move_window:SetVisible(moving)
	self.move_window:SetBackColor(Turbine.UI.Color(0.3,0.3,0.3))
	self.move_window:SetOpacity(0.8)

    self.move_label = Turbine.UI.Label()
	self.move_label:SetParent(self.move_window)
	self.move_label:SetPosition(1,1)
	self.move_label:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
	self.move_label:SetFont(Turbine.UI.Lotro.Font.Verdana12)
    self.move_label:SetFontStyle(Turbine.UI.FontStyle.Outline)
    self.move_label:SetText(self.data["name"])
    self.move_label:SetMouseVisible(false)
    self.move_label:SetBackColor(Turbine.UI.Color.Black)
    self.move_label:SetForeColor(Turbine.UI.Color.White)
    
    self.move_window.MouseDown = function( sender, args )
		if args.Button == Turbine.UI.MouseButton.Left then
			self.dragging = true	
			self.dragStartX = args.X
			self.dragStartY = args.Y
			move_selection_Changed(self)
		end
	end
	
	self.move_window.MouseMove = function( sender, args )
		if self.dragging then
			local x,y = self:GetPosition()	
			x = x + ( args.X - self.dragStartX )
			y = y + ( args.Y - self.dragStartY )	
			self.data["left"],self.data["top"] = self:GetPosition()
			self:SetPosition( x, y )	
			move_selection_Moved()
		end
	end
	
	self.move_window.MouseUp = function( sender, args )
		if args.Button == Turbine.UI.MouseButton.Left then
			self.dragging = false
			self.data["left"],self.data["top"] = self:GetPosition()
				
			SaveSettings()
		end
	end
	
    self:Set_Size(width, height)

end

function Icon_Window:LeftChanged(left)

    self.data["left"] = left
	self:SetLeft(left)
    move_selection_Moved()
    SaveSettings()

end

function Icon_Window:TopChanged(top)

    self.data["top"] = top
	self:SetTop(top)
    move_selection_Moved()
    SaveSettings()

end

function Icon_Window:WidthChanged(width)

    self.data["width"] = width
    self.move_window:SetWidth(width)
    self.move_label:SetWidth(width-2)
	self:SetWidth(width)
    SaveSettings()

    for key, control in ipairs(self.controls) do
        control:WidthChanged(width)
    end

end

function Icon_Window:HeightChanged(height)

    self.data["height"] = height
    self.move_window:SetHeight(height)
    self.move_label:SetHeight(height-2)
	self:SetHeight(height)
    SaveSettings()

    for key, control in ipairs(self.controls) do
        control:HeightChanged(height)
    end

end


function Icon_Window:GroupMove(x_diff, y_diff)

    self.data["left"] = self.data["left"] + x_diff
    self.data["top"] = self.data["top"] + y_diff

    self:SetPosition(self.data["left"], self.data["top"])

end


function Icon_Window:GetLeftForGroup()
    return self:GetLeft()
end

function Icon_Window:GetTopForGroup()
    return self:GetTop()
end

function Icon_Window:GetRightForGroup()
    return self:GetLeft() + self:GetWidth()
end

function Icon_Window:GetBottomForGroup()
    return self:GetTop() + self:GetHeight()
end