FLASHINGSPEED = 2


Bar_W = class( Turbine.UI.Window )

function Bar_W:Constructor( parent, key, start_time, duration, icon, text, data, running )
	Turbine.UI.Window.Constructor( self )

    self.parent = parent
    self.data = data

    self.key = key
    self.start_time = start_time
	self.duration = duration
	if  not(type(self.duration) == "number") then
		self.parent:DeleteItem(self)
		return
	end
    self.endtime = start_time + duration

    self.threshold = self.data["threshold"]
	self.flashing = self.data["flashing"]

	self.opacity2 = self.parent.data["opacity2"]
	if self.opacity2 == nil then
		self.opacity2 = 1
	end

	if  not(type(self.threshold) == "number") then
		self.threshold = 3
		Turbine.Shell.WriteLine("Gibberish: "..text.." is missing a threshold.")
	end

	self.first_in_threshold = true

	self.width = self.parent.data["width"]
    self.back_color = Turbine.UI.Color(self.parent.data["back_color"]["R"], self.parent.data["back_color"]["G"], self.parent.data["back_color"]["B"])

	self:Draw(icon)
	self.text_label:SetText(text)

	if running then
		self:Activ()
	else
        self:Passiv()
	end
	
	self:SetVisible(true)

end


---------------------------------------------------------------------
--== public ===--
---------------------------------------------------------------------

function Bar_W:GetTriggerInfo()

	if self:GetWantsUpdates() == true and (self.data["type"] == CHAT or self.data["type"] == TRIGGER_END or self.data["type"] == TRIGGER_START or self.data["type"] == TRIGGER_THRESHOLD) then
		local list = {}

		list["key"] = self.key
		list["start_time"] = self.start_time
		list["duration"] = self.duration
		list["text"] = self.text_label:GetText()
		list["data"] = self.data

		return list
	else
		return nil
	end

end


function Bar_W:Delete()

	self:SetVisible(false)
	self:SetWantsUpdates(false)
	self.bar_background:SetVisible(false)
	self.background:SetVisible(false)
	self.label_background:SetVisible(false)
	self.icon:SetVisible(false)

	self.label_background:Close()
	self.bar_background:Close()
	self:Close()
end

function Bar_W:Reset(start_time, duration, icon, text)

	if icon ~= nil then --self.icon_set == false and 
		self.icon:SetBackColor()
		self.icon:SetSize(32,32)
		self.icon:SetStretchMode(1)
		self.icon:SetBackground(icon)
		self.icon:SetSize(self.parent.data["height"], self.parent.data["height"])	
		--self.icon_set = true
	end

	if self.data["protected"] then

		if self.endtime >= Turbine.Engine.GetGameTime() then
			return
		end

	end
	self:Passiv()

	if text ~= nil then
		self.text_label:SetText(text)
	end
	self.start_time = start_time
	self.duration = duration
	self.endtime = start_time + self.duration
	self.first_in_threshold = true

	self:Activ()


end

function Bar_W:UpdateSize()

	self.width = self.parent.data["width"]

	local frame = self.parent.data["frame"]
	local height =self.parent.data["height"]
	local full_width = self.width + (3*frame) + height
	local full_height = height + (2*frame) + self.parent.data["spacing"]

	self:SetSize(full_width, full_height)
	self.frame:SetSize(full_width, full_height -  self.parent.data["spacing"])
	self.icon:SetSize(height, height)	
	self.background:SetSize(self.width, height)
	self.bar_background:SetSize(self.width, height)
	self.label_background:SetWidth(self.width-3)
	self.label_background:SetHeight(height)
	self.text_label:SetHeight(self.label_background:GetHeight())
	self.text_label:SetWidth(self.label_background:GetWidth() - 40)
	self.timer_label:SetSize(self.label_background:GetSize())

	self.background:SetPosition(height + (2*frame), frame)
	self.bar_background:SetPosition(height + (2*frame), frame)
	self.label_background:SetLeft(self.bar_background:GetLeft()+1)

end

---------------------------------------------------------------------
--== private ===--
---------------------------------------------------------------------

function Bar_W:Trigger_Ending()
	Trigger.check_for_waiting_trigger_end(self.data["id"]..self.key)
end

function Bar_W:Trigger_Starting()
	Trigger.check_for_waiting_trigger_start(self.data["id"]..self.key)
end

function Bar_W:Trigger_Threshold()
	Trigger.check_for_waiting_trigger_threshold(self.data["id"]..self.key)
end

function Bar_W:ShutDown()

    self:Trigger_Ending()

    if self.data["loop"] then

		self:Activ()

        self.background:SetBackColor(self.back_color)
        self.timer_label:SetText("")

        self.start_time = Turbine.Engine.GetGameTime()
		self.endtime = self.start_time + self.duration


    else
        self.background:SetBackColor(self.back_color)
        self.timer_label:SetText("")
        self.bar:SetWidth(0)
        self:Passiv()
        self.parent:DeleteItem(self)
    end

end

function Bar_W:Update()
	local game_time = Turbine.Engine.GetGameTime()
	local time_left = self.endtime - game_time

	if time_left < 0 then
		self:ShutDown()
		return
	end

	if time_left < 99999 then
		self.timer_label:SetText(SecondsToClock(time_left))
		self.bar:SetWidth(time_left/self.duration*self.width)
	else
		self.timer_label:SetText("")
		self.bar:SetWidth(0)
	end

	if(time_left < self.threshold ) then

		if self.first_in_threshold then
			self:Trigger_Threshold()
			self.first_in_threshold = false
		end

		if(self.flashing) then
			local value
			local flash_value = time_left*FLASHINGSPEED
			if math.floor(flash_value) % 2 == 0 then
				value = 1-(flash_value-math.floor(flash_value))
			else
				value = (flash_value-math.floor(flash_value))
			end

			
			self.background:SetBackColor(Turbine.UI.Color(1, value, value))
		else
			self.background:SetBackColor(Turbine.UI.Color.Red)
		end

	end
end

function Bar_W:Passiv()

	self:SetWantsUpdates(false)
	self.bar:SetWidth(0)
	self.timer_label:SetText("")
    self.label_background:SetOpacity(self.parent.data["opacity"])
    self.icon:SetOpacity(self.parent.data["opacity"])
end

function Bar_W:Activ()

	self:SetWantsUpdates(true)
	self.background:SetBackColor(self.back_color)
	self.label_background:SetOpacity(self.opacity2)
	self.icon:SetOpacity(self.opacity2)

end

function Bar_W:Draw(icon)

    local frame = self.parent.data["frame"]
	local height =self.parent.data["height"]
	local full_width = self.width + (3*frame) + height
	local full_height = height + (2*frame) + self.parent.data["spacing"]

    self.icon = Turbine.UI.Control()
	if icon ~= nil then
		self.icon:SetMouseVisible(false)
		self.icon:SetPosition(frame,frame + self.parent.data["spacing"])
		self.icon:SetSize(32,32)
		self.icon:SetStretchMode(1)
		self.icon:SetBackground(tonumber(icon))
		self.icon:SetZOrder(4)
		--self.icon_set = true
	else
		self.icon:SetMouseVisible(false)
		self.icon:SetPosition(frame,frame + self.parent.data["spacing"])
		self.icon:SetBackColor(self.back_color)
		self.icon:SetZOrder(4)
	end

	self:SetSize(full_width, full_height)
	self:SetMouseVisible(false)
	self:SetOpacity(self.parent.data["opacity"])

	self.icon:SetParent(self)
	self.icon:SetSize(height, height)

	self.frame = Turbine.UI.Control()
	self.frame:SetParent(self)
	self.frame:SetTop( self.parent.data["spacing"])
	self.frame:SetSize(full_width, full_height -  self.parent.data["spacing"])
	self.frame:SetMouseVisible(false)
	self.frame:SetBackColor(Turbine.UI.Color(self.parent.data["frame_color"]["R"], self.parent.data["frame_color"]["G"], self.parent.data["frame_color"]["B"]))
	self.frame:SetZOrder(2)

	self.background = Turbine.UI.Control()
	self.background:SetParent(self.frame)
	self.background:SetMouseVisible(false)
	self.background:SetPosition(height + (2*frame), frame)
	self.background:SetSize(self.width, height)
	self.background:SetBackColor(self.back_color)
	self.background:SetZOrder(4)
	self.background:SetVisible(true)
	self.background:SetOpacity(self.parent.data["opacity"])

	self.bar_background = Turbine.UI.Window()
	self.bar_background:SetParent(self.frame)
	self.bar_background:SetMouseVisible(false)
	self.bar_background:SetPosition(height + (2*frame), frame)
	self.bar_background:SetSize(self.width, height)
	self.bar_background:SetZOrder(5)
	self.bar_background:SetVisible(true)

	self.bar = Turbine.UI.Control()
	self.bar:SetParent(self.bar_background)
	self.bar:SetMouseVisible(false)
	self.bar:SetWidth(0)
	self.bar:SetPosition(0,0)
	self.bar:SetHeight(height)
	self.bar:SetBackColor(Turbine.UI.Color(self.parent.data["bar_color"]["R"], self.parent.data["bar_color"]["G"], self.parent.data["bar_color"]["B"]))
	self.bar:SetVisible(true)
	self.bar:SetZOrder(8)

	self.label_background = Turbine.UI.Window()
	self.label_background:SetParent(self.frame)
	self.label_background:SetMouseVisible(false)
	self.label_background:SetLeft(self.bar_background:GetLeft()+1)
	self.label_background:SetTop(frame)
	self.label_background:SetWidth(self.width-3)
	self.label_background:SetHeight(height)
	self.label_background:SetZOrder(10)
	self.label_background:SetVisible(true)


	self.text_label = Turbine.UI.Label()
	self.text_label:SetParent(self.label_background)
	self.text_label:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft)
	self.text_label:SetMouseVisible(false)
	self.text_label:SetFontStyle(Turbine.UI.FontStyle.Outline)
	self.text_label:SetWidth(self.label_background:GetWidth() - 40)
	self.text_label:SetHeight(self.label_background:GetHeight())
	self.text_label:SetFont(findfont(self.parent.data["font"]))
	self.text_label:SetText(text)
	self.text_label:SetZOrder(12)

	self.timer_label = Turbine.UI.Label()
	self.timer_label:SetParent(self.label_background)
	self.timer_label:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleRight)
	self.timer_label:SetMouseVisible(false)
	self.timer_label:SetFontStyle(Turbine.UI.FontStyle.Outline)
	self.timer_label:SetSize(self.label_background:GetSize())
	self.timer_label:SetFont(findfont(self.parent.data["font"]))
	self.timer_label:SetText("")
	self.timer_label:SetZOrder(14)

	if self.data["hide_text"] then
		self.text_label:SetVisible(false)
		self.timer_label:SetVisible(false)
	end

end