
Icon = class( Turbine.UI.Window )

function Icon:Constructor( parent, key, start_time, duration, icon, text, data, running )
    Turbine.UI.Window.Constructor( self )

    self.parent = parent
    self.data = data

    self.key = key
    self.start_time = start_time
	self.duration = duration
	if  not(type(self.duration) == "number") then
		self.parent:DeleteItem(self)
		return
	end
    self.endtime = start_time + duration

	self.threshold = self.data["threshold"]
	self.popup_multi = self.data["popup_multi"]
	self.flashing = self.data["flashing"]

	self.opacity2 = self.parent.data["opacity2"]
	if self.opacity2 == nil then
		self.opacity2 = 1
	end

	if  not(type(self.threshold) == "number") then
		self.threshold = 3
		self.data["threshold"] = 3
	end
	
	if  not(type(self.popup_multi) == "number") then
		self.popup_multi = 10
		self.data["popup_multi"] = 10
    end

	self.first_in_threshold = true


	self.width = self.parent.data["width"]
    self.back_color = Turbine.UI.Color(self.parent.data["back_color"]["R"], self.parent.data["back_color"]["G"], self.parent.data["back_color"]["B"])

	self:Draw(icon)
	self.text_label:SetText(text)

	if running then
		self:Activ()
	else
        self:Passiv()
	end
	
	self:SetVisible(true)


end

---------------------------------------------------------------------
--== public ===--
---------------------------------------------------------------------

function Icon:GetTriggerInfo()
	if self:GetWantsUpdates() == true and (self.data["type"] == CHAT or self.data["type"] == TRIGGER_END or self.data["type"] == TRIGGER_START or self.data["type"] == TRIGGER_THRESHOLD) then
		local list = {}

		list["key"] = self.key
		list["start_time"] = self.start_time
		list["duration"] = self.duration
		--list["text"] = self.text_label:GetText()
		list["data"] = self.data

		return list
	else
		return nil
	end
end

function Icon:Delete()
	self:SetVisible(false)
	self.label_background:SetVisible(false)
    self.icon:SetVisible(false)

	self.label_background:Close()
	self:Close()
end

function Icon:Reset(start_time, duration, icon, text)

    if icon ~= nil and icon ~= 0 then
		self.icon:SetBackColor()
		self.icon:SetSize(32,32)
		self.icon:SetStretchMode(1)
		self.icon:SetBackground(icon)
        self.icon:SetSize(self.parent.data["width"], self.parent.data["width"])	
	end
	

	if self.data["protected"] == false or self:GetWantsUpdates() == false then
		self:Passiv()

		self.text_label:SetText(text)
		
		self.start_time = start_time
		self.duration = duration
		self.endtime = start_time + self.duration
		self.first_in_threshold = true

		self:Activ()
	end

end

function Icon:WidthChanged(width)
	self.width = width

	local height =self.parent.data["width"]
	local full_width = self.width + self.parent.data["spacing"]
	local full_height = height + self.parent.data["spacing"]

    self:SetSize(full_width, full_height)
	self.icon:SetSize(self.width, height)	
	self.label_background:SetWidth(self.width )
	self.text_label:SetWidth(self.label_background:GetWidth())
	self.timer_label:SetSize(self.label_background:GetSize())


end

function Icon:HeightChanged(height)

	local height =self.parent.data["width"]
	local full_width = self.width + self.parent.data["spacing"]
	local full_height = height + self.parent.data["spacing"]

    self:SetSize(full_width, full_height)
	self.icon:SetSize(self.width, height)	
	self.label_background:SetHeight(height)
	self.text_label:SetHeight(self.label_background:GetHeight())
	self.timer_label:SetSize(self.label_background:GetSize())


end

---------------------------------------------------------------------
--== private ===--
---------------------------------------------------------------------

function Icon:Trigger_Ending()
	Trigger.check_for_waiting_trigger_end(self.data["id"]..self.key)
end

function Icon:Trigger_Starting()
	Trigger.check_for_waiting_trigger_start(self.data["id"]..self.key)
end

function Icon:Trigger_Threshold()
	Trigger.check_for_waiting_trigger_threshold(self.data["id"]..self.key)
end


function Icon:ShutDown()

    self:Trigger_Ending()

    if self.data["loop"] then

		self:Activ()

        self.timer_label:SetText("")
        self.timer_label:SetForeColor(Turbine.UI.Color.White)

        self.start_time = Turbine.Engine.GetGameTime()
		self.endtime = self.start_time + self.duration


    else
        self.timer_label:SetText("")
        self.timer_label:SetForeColor(Turbine.UI.Color.White)
        self:Passiv()
        self.parent:DeleteItem(self)
    end

end


function Icon:Update()

	local game_time = Turbine.Engine.GetGameTime()
	local time_left = self.endtime - game_time

	if time_left < 0 then
		self:ShutDown()
		return
    end

    if time_left < 99999 then
		self.timer_label:SetText(math.round(time_left))
	else
		self.timer_label:SetText("")
    end
    
    if self.flashing and (time_left < self.threshold ) then

        if self.first_in_threshold then
			self:Trigger_Threshold()
            self.first_in_threshold = false
        end
		
		-- local size = (self.width * time_left * self.popup_multi) + self.width
		-- local pos = (self.width * self.popup_multi * time_left * (-1) / 2)

		local size = (time_left / self.threshold * (self.popup_multi - 1) + 1) * self.width
        local pos = (time_left / self.threshold * (self.popup_multi - 1)) * self.width * (-1)/2

		self.icon:SetPosition(pos, pos)
		self.icon:SetSize(size, size)
		self.icon:SetOpacity(self.parent.data["opacity"]+(1-self.parent.data["opacity"])*(1-(time_left/self.threshold)))


    end

end

function Icon:Passiv()

	self:SetWantsUpdates(false)
	self.timer_label:SetText("")
	self.text_label:SetText("")
	self.icon:SetSize(self.width, self.width)
	self.timer_label:SetForeColor(Turbine.UI.Color.White)
	self.icon:SetPosition(0, 0)
    self.icon:SetOpacity(self.opacity2)

end

function Icon:Activ()

	self:SetWantsUpdates(true)
    self.icon:SetOpacity(self.parent.data["opacity"])

end

function Icon:Draw(icon)

	local height =self.parent.data["width"]
	local full_width = self.width + self.parent.data["spacing"]
	local full_height = height + self.parent.data["spacing"]

    self.icon = Turbine.UI.Control()

	if icon == 0 then

		self.icon:SetMouseVisible(false)
		self.icon:SetPosition(0, 0)
        self.icon:SetZOrder(4)

    elseif icon ~= nil then
		self.icon:SetMouseVisible(false)
		self.icon:SetPosition(0, 0)
		self.icon:SetSize(32,32)
		self.icon:SetStretchMode(1)
		self.icon:SetBackground(tonumber(icon))
		self.icon:SetZOrder(4)

	else

		self.icon:SetMouseVisible(false)
		self.icon:SetPosition(0, 0)
		self.icon:SetBackColor(self.back_color)
        self.icon:SetZOrder(4)

    end
    
    self:SetSize(full_width, full_height)
	self:SetMouseVisible(false)

    self.icon:SetParent(self)
	self.icon:SetSize(self.width, height)	

    self.label_background = Turbine.UI.Window()
    self.label_background:SetParent(self)
	self.label_background:SetMouseVisible(false)
	self.label_background:SetLeft(0)
	self.label_background:SetTop(0)
	self.label_background:SetWidth(self.width )
	self.label_background:SetHeight(height)
	self.label_background:SetZOrder(10)
    self.label_background:SetVisible(true)

	self.text_label = Turbine.UI.Label()
	self.text_label:SetParent(self.label_background)
	self.text_label:SetTextAlignment(Turbine.UI.ContentAlignment.TopLeft)
	self.text_label:SetMouseVisible(false)
	self.text_label:SetTop(-2)
	self.text_label:SetForeColor(Turbine.UI.Color(0.5,0.5,0.5))
	self.text_label:SetFontStyle(Turbine.UI.FontStyle.Outline)
	self.text_label:SetWidth(self.label_background:GetWidth())
	self.text_label:SetHeight(self.label_background:GetHeight())
	self.text_label:SetFont(findfont(self.parent.data["font"]))
	self.text_label:SetZOrder(12)

    self.timer_label = Turbine.UI.Label()
    self.timer_label:SetParent(self.label_background)
	self.timer_label:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
	self.timer_label:SetMouseVisible(false)
	self.timer_label:SetFontStyle(Turbine.UI.FontStyle.Outline)
	self.timer_label:SetSize(self.label_background:GetSize())
	self.timer_label:SetFont(findfont(self.parent.data["font"]))
	self.timer_label:SetText("")

	if self.data["hide_text"] then
		self.text_label:SetVisible(false)
		self.timer_label:SetVisible(false)
	end

end